# DO NOT EDIT! This file is automatically generated
import marshmallow
import marshmallow_enum

from commercetools import helpers, types
from commercetools._schemas._common import (
    BaseResourceSchema,
    LocalizedStringField,
    ReferenceSchema,
    ResourceIdentifierSchema,
)

__all__ = [
    "CartDiscountChangeCartPredicateActionSchema",
    "CartDiscountChangeIsActiveActionSchema",
    "CartDiscountChangeNameActionSchema",
    "CartDiscountChangeRequiresDiscountCodeActionSchema",
    "CartDiscountChangeSortOrderActionSchema",
    "CartDiscountChangeStackingModeActionSchema",
    "CartDiscountChangeTargetActionSchema",
    "CartDiscountChangeValueActionSchema",
    "CartDiscountCustomLineItemsTargetSchema",
    "CartDiscountDraftSchema",
    "CartDiscountLineItemsTargetSchema",
    "CartDiscountPagedQueryResponseSchema",
    "CartDiscountReferenceSchema",
    "CartDiscountResourceIdentifierSchema",
    "CartDiscountSchema",
    "CartDiscountSetCustomFieldActionSchema",
    "CartDiscountSetCustomTypeActionSchema",
    "CartDiscountSetDescriptionActionSchema",
    "CartDiscountSetKeyActionSchema",
    "CartDiscountSetValidFromActionSchema",
    "CartDiscountSetValidFromAndUntilActionSchema",
    "CartDiscountSetValidUntilActionSchema",
    "CartDiscountShippingCostTargetSchema",
    "CartDiscountTargetSchema",
    "CartDiscountUpdateActionSchema",
    "CartDiscountUpdateSchema",
    "CartDiscountValueAbsoluteDraftSchema",
    "CartDiscountValueAbsoluteSchema",
    "CartDiscountValueDraftSchema",
    "CartDiscountValueGiftLineItemDraftSchema",
    "CartDiscountValueGiftLineItemSchema",
    "CartDiscountValueRelativeDraftSchema",
    "CartDiscountValueRelativeSchema",
    "CartDiscountValueSchema",
    "MultiBuyCustomLineItemsTargetSchema",
    "MultiBuyLineItemsTargetSchema",
]


class CartDiscountDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountDraft`."""

    name = LocalizedStringField(allow_none=True)
    key = marshmallow.fields.String(allow_none=True, missing=None)
    description = LocalizedStringField(allow_none=True, missing=None)
    value = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "absolute": "commercetools._schemas._cart_discount.CartDiscountValueAbsoluteDraftSchema",
            "giftLineItem": "commercetools._schemas._cart_discount.CartDiscountValueGiftLineItemDraftSchema",
            "relative": "commercetools._schemas._cart_discount.CartDiscountValueRelativeDraftSchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    cart_predicate = marshmallow.fields.String(
        allow_none=True, data_key="cartPredicate"
    )
    target = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "customLineItems": "commercetools._schemas._cart_discount.CartDiscountCustomLineItemsTargetSchema",
            "lineItems": "commercetools._schemas._cart_discount.CartDiscountLineItemsTargetSchema",
            "shipping": "commercetools._schemas._cart_discount.CartDiscountShippingCostTargetSchema",
            "multiBuyCustomLineItems": "commercetools._schemas._cart_discount.MultiBuyCustomLineItemsTargetSchema",
            "multiBuyLineItems": "commercetools._schemas._cart_discount.MultiBuyLineItemsTargetSchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    sort_order = marshmallow.fields.String(allow_none=True, data_key="sortOrder")
    is_active = marshmallow.fields.Bool(
        allow_none=True, missing=None, data_key="isActive"
    )
    valid_from = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="validFrom"
    )
    valid_until = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="validUntil"
    )
    requires_discount_code = marshmallow.fields.Bool(
        allow_none=True, data_key="requiresDiscountCode"
    )
    stacking_mode = marshmallow_enum.EnumField(
        types.StackingMode, by_value=True, missing=None, data_key="stackingMode"
    )
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.CartDiscountDraft(**data)


class CartDiscountPagedQueryResponseSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountPagedQueryResponse`."""

    limit = marshmallow.fields.Integer(allow_none=True)
    count = marshmallow.fields.Integer(allow_none=True)
    total = marshmallow.fields.Integer(allow_none=True, missing=None)
    offset = marshmallow.fields.Integer(allow_none=True)
    results = helpers.LazyNestedField(
        nested="commercetools._schemas._cart_discount.CartDiscountSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.CartDiscountPagedQueryResponse(**data)


class CartDiscountReferenceSchema(ReferenceSchema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountReference`."""

    obj = helpers.LazyNestedField(
        nested="commercetools._schemas._cart_discount.CartDiscountSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.CartDiscountReference(**data)


class CartDiscountResourceIdentifierSchema(ResourceIdentifierSchema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountResourceIdentifier`."""

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.CartDiscountResourceIdentifier(**data)


class CartDiscountSchema(BaseResourceSchema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscount`."""

    id = marshmallow.fields.String(allow_none=True)
    version = marshmallow.fields.Integer(allow_none=True)
    created_at = marshmallow.fields.DateTime(allow_none=True, data_key="createdAt")
    last_modified_at = marshmallow.fields.DateTime(
        allow_none=True, data_key="lastModifiedAt"
    )
    last_modified_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.LastModifiedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="lastModifiedBy",
    )
    created_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.CreatedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="createdBy",
    )
    name = LocalizedStringField(allow_none=True)
    key = marshmallow.fields.String(allow_none=True, missing=None)
    description = LocalizedStringField(allow_none=True, missing=None)
    value = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "absolute": "commercetools._schemas._cart_discount.CartDiscountValueAbsoluteSchema",
            "giftLineItem": "commercetools._schemas._cart_discount.CartDiscountValueGiftLineItemSchema",
            "relative": "commercetools._schemas._cart_discount.CartDiscountValueRelativeSchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    cart_predicate = marshmallow.fields.String(
        allow_none=True, data_key="cartPredicate"
    )
    target = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "customLineItems": "commercetools._schemas._cart_discount.CartDiscountCustomLineItemsTargetSchema",
            "lineItems": "commercetools._schemas._cart_discount.CartDiscountLineItemsTargetSchema",
            "shipping": "commercetools._schemas._cart_discount.CartDiscountShippingCostTargetSchema",
            "multiBuyCustomLineItems": "commercetools._schemas._cart_discount.MultiBuyCustomLineItemsTargetSchema",
            "multiBuyLineItems": "commercetools._schemas._cart_discount.MultiBuyLineItemsTargetSchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    sort_order = marshmallow.fields.String(allow_none=True, data_key="sortOrder")
    is_active = marshmallow.fields.Bool(allow_none=True, data_key="isActive")
    valid_from = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="validFrom"
    )
    valid_until = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="validUntil"
    )
    requires_discount_code = marshmallow.fields.Bool(
        allow_none=True, data_key="requiresDiscountCode"
    )
    references = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("typeId", "type_id"),
            discriminator_schemas={
                "cart-discount": "commercetools._schemas._cart_discount.CartDiscountReferenceSchema",
                "cart": "commercetools._schemas._cart.CartReferenceSchema",
                "category": "commercetools._schemas._category.CategoryReferenceSchema",
                "channel": "commercetools._schemas._channel.ChannelReferenceSchema",
                "key-value-document": "commercetools._schemas._custom_object.CustomObjectReferenceSchema",
                "customer-group": "commercetools._schemas._customer_group.CustomerGroupReferenceSchema",
                "customer": "commercetools._schemas._customer.CustomerReferenceSchema",
                "discount-code": "commercetools._schemas._discount_code.DiscountCodeReferenceSchema",
                "inventory-entry": "commercetools._schemas._inventory.InventoryEntryReferenceSchema",
                "order-edit": "commercetools._schemas._order_edit.OrderEditReferenceSchema",
                "order": "commercetools._schemas._order.OrderReferenceSchema",
                "payment": "commercetools._schemas._payment.PaymentReferenceSchema",
                "product-discount": "commercetools._schemas._product_discount.ProductDiscountReferenceSchema",
                "product-type": "commercetools._schemas._product_type.ProductTypeReferenceSchema",
                "product": "commercetools._schemas._product.ProductReferenceSchema",
                "review": "commercetools._schemas._review.ReviewReferenceSchema",
                "shipping-method": "commercetools._schemas._shipping_method.ShippingMethodReferenceSchema",
                "shopping-list": "commercetools._schemas._shopping_list.ShoppingListReferenceSchema",
                "state": "commercetools._schemas._state.StateReferenceSchema",
                "store": "commercetools._schemas._store.StoreReferenceSchema",
                "tax-category": "commercetools._schemas._tax_category.TaxCategoryReferenceSchema",
                "type": "commercetools._schemas._type.TypeReferenceSchema",
                "zone": "commercetools._schemas._zone.ZoneReferenceSchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        )
    )
    stacking_mode = marshmallow_enum.EnumField(
        types.StackingMode, by_value=True, data_key="stackingMode"
    )
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.CartDiscount(**data)


class CartDiscountTargetSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountTarget`."""

    type = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.CartDiscountTarget(**data)


class CartDiscountUpdateActionSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountUpdateAction`."""

    action = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartDiscountUpdateAction(**data)


class CartDiscountUpdateSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountUpdate`."""

    version = marshmallow.fields.Integer(allow_none=True)
    actions = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("action", "action"),
            discriminator_schemas={
                "changeCartPredicate": "commercetools._schemas._cart_discount.CartDiscountChangeCartPredicateActionSchema",
                "changeIsActive": "commercetools._schemas._cart_discount.CartDiscountChangeIsActiveActionSchema",
                "changeName": "commercetools._schemas._cart_discount.CartDiscountChangeNameActionSchema",
                "changeRequiresDiscountCode": "commercetools._schemas._cart_discount.CartDiscountChangeRequiresDiscountCodeActionSchema",
                "changeSortOrder": "commercetools._schemas._cart_discount.CartDiscountChangeSortOrderActionSchema",
                "changeStackingMode": "commercetools._schemas._cart_discount.CartDiscountChangeStackingModeActionSchema",
                "changeTarget": "commercetools._schemas._cart_discount.CartDiscountChangeTargetActionSchema",
                "changeValue": "commercetools._schemas._cart_discount.CartDiscountChangeValueActionSchema",
                "setCustomField": "commercetools._schemas._cart_discount.CartDiscountSetCustomFieldActionSchema",
                "setCustomType": "commercetools._schemas._cart_discount.CartDiscountSetCustomTypeActionSchema",
                "setDescription": "commercetools._schemas._cart_discount.CartDiscountSetDescriptionActionSchema",
                "setKey": "commercetools._schemas._cart_discount.CartDiscountSetKeyActionSchema",
                "setValidFrom": "commercetools._schemas._cart_discount.CartDiscountSetValidFromActionSchema",
                "setValidFromAndUntil": "commercetools._schemas._cart_discount.CartDiscountSetValidFromAndUntilActionSchema",
                "setValidUntil": "commercetools._schemas._cart_discount.CartDiscountSetValidUntilActionSchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        ),
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.CartDiscountUpdate(**data)


class CartDiscountValueDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountValueDraft`."""

    type = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.CartDiscountValueDraft(**data)


class CartDiscountValueSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountValue`."""

    type = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.CartDiscountValue(**data)


class CartDiscountChangeCartPredicateActionSchema(CartDiscountUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountChangeCartPredicateAction`."""

    cart_predicate = marshmallow.fields.String(
        allow_none=True, data_key="cartPredicate"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartDiscountChangeCartPredicateAction(**data)


class CartDiscountChangeIsActiveActionSchema(CartDiscountUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountChangeIsActiveAction`."""

    is_active = marshmallow.fields.Bool(allow_none=True, data_key="isActive")

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartDiscountChangeIsActiveAction(**data)


class CartDiscountChangeNameActionSchema(CartDiscountUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountChangeNameAction`."""

    name = LocalizedStringField(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartDiscountChangeNameAction(**data)


class CartDiscountChangeRequiresDiscountCodeActionSchema(
    CartDiscountUpdateActionSchema
):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountChangeRequiresDiscountCodeAction`."""

    requires_discount_code = marshmallow.fields.Bool(
        allow_none=True, data_key="requiresDiscountCode"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartDiscountChangeRequiresDiscountCodeAction(**data)


class CartDiscountChangeSortOrderActionSchema(CartDiscountUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountChangeSortOrderAction`."""

    sort_order = marshmallow.fields.String(allow_none=True, data_key="sortOrder")

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartDiscountChangeSortOrderAction(**data)


class CartDiscountChangeStackingModeActionSchema(CartDiscountUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountChangeStackingModeAction`."""

    stacking_mode = marshmallow_enum.EnumField(
        types.StackingMode, by_value=True, data_key="stackingMode"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartDiscountChangeStackingModeAction(**data)


class CartDiscountChangeTargetActionSchema(CartDiscountUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountChangeTargetAction`."""

    target = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "customLineItems": "commercetools._schemas._cart_discount.CartDiscountCustomLineItemsTargetSchema",
            "lineItems": "commercetools._schemas._cart_discount.CartDiscountLineItemsTargetSchema",
            "shipping": "commercetools._schemas._cart_discount.CartDiscountShippingCostTargetSchema",
            "multiBuyCustomLineItems": "commercetools._schemas._cart_discount.MultiBuyCustomLineItemsTargetSchema",
            "multiBuyLineItems": "commercetools._schemas._cart_discount.MultiBuyLineItemsTargetSchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartDiscountChangeTargetAction(**data)


class CartDiscountChangeValueActionSchema(CartDiscountUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountChangeValueAction`."""

    value = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "absolute": "commercetools._schemas._cart_discount.CartDiscountValueAbsoluteDraftSchema",
            "giftLineItem": "commercetools._schemas._cart_discount.CartDiscountValueGiftLineItemDraftSchema",
            "relative": "commercetools._schemas._cart_discount.CartDiscountValueRelativeDraftSchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartDiscountChangeValueAction(**data)


class CartDiscountCustomLineItemsTargetSchema(CartDiscountTargetSchema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountCustomLineItemsTarget`."""

    predicate = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.CartDiscountCustomLineItemsTarget(**data)


class CartDiscountLineItemsTargetSchema(CartDiscountTargetSchema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountLineItemsTarget`."""

    predicate = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.CartDiscountLineItemsTarget(**data)


class CartDiscountSetCustomFieldActionSchema(CartDiscountUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountSetCustomFieldAction`."""

    name = marshmallow.fields.String(allow_none=True)
    value = marshmallow.fields.Raw(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartDiscountSetCustomFieldAction(**data)


class CartDiscountSetCustomTypeActionSchema(CartDiscountUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountSetCustomTypeAction`."""

    type = helpers.LazyNestedField(
        nested="commercetools._schemas._type.TypeResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    fields = marshmallow.fields.Dict(allow_none=True, missing=None)  # type: ignore

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartDiscountSetCustomTypeAction(**data)


class CartDiscountSetDescriptionActionSchema(CartDiscountUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountSetDescriptionAction`."""

    description = LocalizedStringField(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartDiscountSetDescriptionAction(**data)


class CartDiscountSetKeyActionSchema(CartDiscountUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountSetKeyAction`."""

    key = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartDiscountSetKeyAction(**data)


class CartDiscountSetValidFromActionSchema(CartDiscountUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountSetValidFromAction`."""

    valid_from = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="validFrom"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartDiscountSetValidFromAction(**data)


class CartDiscountSetValidFromAndUntilActionSchema(CartDiscountUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountSetValidFromAndUntilAction`."""

    valid_from = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="validFrom"
    )
    valid_until = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="validUntil"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartDiscountSetValidFromAndUntilAction(**data)


class CartDiscountSetValidUntilActionSchema(CartDiscountUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountSetValidUntilAction`."""

    valid_until = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="validUntil"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.CartDiscountSetValidUntilAction(**data)


class CartDiscountShippingCostTargetSchema(CartDiscountTargetSchema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountShippingCostTarget`."""

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.CartDiscountShippingCostTarget()


class CartDiscountValueAbsoluteDraftSchema(CartDiscountValueDraftSchema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountValueAbsoluteDraft`."""

    money = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.CartDiscountValueAbsoluteDraft(**data)


class CartDiscountValueAbsoluteSchema(CartDiscountValueSchema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountValueAbsolute`."""

    money = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("type", "type"),
            discriminator_schemas={
                "centPrecision": "commercetools._schemas._common.CentPrecisionMoneySchema",
                "highPrecision": "commercetools._schemas._common.HighPrecisionMoneySchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        )
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.CartDiscountValueAbsolute(**data)


class CartDiscountValueGiftLineItemDraftSchema(CartDiscountValueDraftSchema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountValueGiftLineItemDraft`."""

    product = helpers.LazyNestedField(
        nested="commercetools._schemas._product.ProductReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    variant_id = marshmallow.fields.Integer(allow_none=True, data_key="variantId")
    supply_channel = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="supplyChannel",
    )
    distribution_channel = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="distributionChannel",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.CartDiscountValueGiftLineItemDraft(**data)


class CartDiscountValueGiftLineItemSchema(CartDiscountValueSchema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountValueGiftLineItem`."""

    product = helpers.LazyNestedField(
        nested="commercetools._schemas._product.ProductReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    variant_id = marshmallow.fields.Integer(allow_none=True, data_key="variantId")
    supply_channel = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="supplyChannel",
    )
    distribution_channel = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="distributionChannel",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.CartDiscountValueGiftLineItem(**data)


class CartDiscountValueRelativeDraftSchema(CartDiscountValueDraftSchema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountValueRelativeDraft`."""

    permyriad = marshmallow.fields.Integer(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.CartDiscountValueRelativeDraft(**data)


class CartDiscountValueRelativeSchema(CartDiscountValueSchema):
    """Marshmallow schema for :class:`commercetools.types.CartDiscountValueRelative`."""

    permyriad = marshmallow.fields.Integer(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.CartDiscountValueRelative(**data)


class MultiBuyCustomLineItemsTargetSchema(CartDiscountTargetSchema):
    """Marshmallow schema for :class:`commercetools.types.MultiBuyCustomLineItemsTarget`."""

    predicate = marshmallow.fields.String(allow_none=True)
    trigger_quantity = marshmallow.fields.Integer(
        allow_none=True, data_key="triggerQuantity"
    )
    discounted_quantity = marshmallow.fields.Integer(
        allow_none=True, data_key="discountedQuantity"
    )
    max_occurrence = marshmallow.fields.Integer(
        allow_none=True, missing=None, data_key="maxOccurrence"
    )
    selection_mode = marshmallow_enum.EnumField(
        types.SelectionMode, by_value=True, data_key="selectionMode"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.MultiBuyCustomLineItemsTarget(**data)


class MultiBuyLineItemsTargetSchema(CartDiscountTargetSchema):
    """Marshmallow schema for :class:`commercetools.types.MultiBuyLineItemsTarget`."""

    predicate = marshmallow.fields.String(allow_none=True)
    trigger_quantity = marshmallow.fields.Integer(
        allow_none=True, data_key="triggerQuantity"
    )
    discounted_quantity = marshmallow.fields.Integer(
        allow_none=True, data_key="discountedQuantity"
    )
    max_occurrence = marshmallow.fields.Integer(
        allow_none=True, missing=None, data_key="maxOccurrence"
    )
    selection_mode = marshmallow_enum.EnumField(
        types.SelectionMode, by_value=True, data_key="selectionMode"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.MultiBuyLineItemsTarget(**data)



# DO NOT EDIT! This file is automatically generated
import datetime
import typing

from commercetools.types._abstract import _BaseType
from commercetools.types._common import (
    BaseResource,
    KeyReference,
    Reference,
    ReferenceTypeId,
    ResourceIdentifier,
)

if typing.TYPE_CHECKING:
    from ._channel import ChannelReference, ChannelResourceIdentifier
    from ._common import CreatedBy, LastModifiedBy, LocalizedString
__all__ = [
    "Store",
    "StoreDraft",
    "StoreKeyReference",
    "StorePagedQueryResponse",
    "StoreReference",
    "StoreResourceIdentifier",
    "StoreSetLanguagesAction",
    "StoreSetNameAction",
    "StoreUpdate",
    "StoreUpdateAction",
    "StoresAddDistributionChannelsAction",
    "StoresAddSupplyChannelsAction",
    "StoresRemoveDistributionChannelsAction",
    "StoresRemoveSupplyChannelsAction",
    "StoresSetDistributionChannelsAction",
    "StoresSetSupplyChannelsAction",
]


class Store(BaseResource):
    #: :class:`str`
    id: str
    #: :class:`int`
    version: int
    #: :class:`datetime.datetime` `(Named` ``createdAt`` `in Commercetools)`
    created_at: datetime.datetime
    #: :class:`datetime.datetime` `(Named` ``lastModifiedAt`` `in Commercetools)`
    last_modified_at: datetime.datetime
    #: Optional :class:`commercetools.types.LastModifiedBy` `(Named` ``lastModifiedBy`` `in Commercetools)`
    last_modified_by: typing.Optional["LastModifiedBy"]
    #: Optional :class:`commercetools.types.CreatedBy` `(Named` ``createdBy`` `in Commercetools)`
    created_by: typing.Optional["CreatedBy"]
    #: :class:`str`
    key: str
    #: Optional :class:`commercetools.types.LocalizedString`
    name: typing.Optional["LocalizedString"]
    #: Optional list of :class:`str`
    languages: typing.Optional[typing.List[str]]
    #: List of :class:`commercetools.types.ChannelReference` `(Named` ``distributionChannels`` `in Commercetools)`
    distribution_channels: typing.List["ChannelReference"]
    #: Optional list of :class:`commercetools.types.ChannelResourceIdentifier` `(Named` ``supplyChannels`` `in Commercetools)`
    supply_channels: typing.Optional[typing.List["ChannelResourceIdentifier"]]

    def __init__(
        self,
        *,
        id: str,
        version: int,
        created_at: datetime.datetime,
        last_modified_at: datetime.datetime,
        key: str,
        distribution_channels: typing.List["ChannelReference"],
        last_modified_by: typing.Optional["LastModifiedBy"] = None,
        created_by: typing.Optional["CreatedBy"] = None,
        name: typing.Optional["LocalizedString"] = None,
        languages: typing.Optional[typing.List[str]] = None,
        supply_channels: typing.Optional[
            typing.List["ChannelResourceIdentifier"]
        ] = None
    ) -> None:
        self.id = id
        self.version = version
        self.created_at = created_at
        self.last_modified_at = last_modified_at
        self.last_modified_by = last_modified_by
        self.created_by = created_by
        self.key = key
        self.name = name
        self.languages = languages
        self.distribution_channels = distribution_channels
        self.supply_channels = supply_channels
        super().__init__(
            id=id,
            version=version,
            created_at=created_at,
            last_modified_at=last_modified_at,
        )

    def __repr__(self) -> str:
        return (
            "Store(id=%r, version=%r, created_at=%r, last_modified_at=%r, last_modified_by=%r, created_by=%r, key=%r, name=%r, languages=%r, distribution_channels=%r, supply_channels=%r)"
            % (
                self.id,
                self.version,
                self.created_at,
                self.last_modified_at,
                self.last_modified_by,
                self.created_by,
                self.key,
                self.name,
                self.languages,
                self.distribution_channels,
                self.supply_channels,
            )
        )


class StoreDraft(_BaseType):
    #: :class:`str`
    key: str
    #: :class:`commercetools.types.LocalizedString`
    name: "LocalizedString"
    #: Optional list of :class:`str`
    languages: typing.Optional[typing.List[str]]
    #: Optional list of :class:`commercetools.types.ChannelResourceIdentifier` `(Named` ``distributionChannels`` `in Commercetools)`
    distribution_channels: typing.Optional[typing.List["ChannelResourceIdentifier"]]
    #: Optional list of :class:`commercetools.types.ChannelResourceIdentifier` `(Named` ``supplyChannels`` `in Commercetools)`
    supply_channels: typing.Optional[typing.List["ChannelResourceIdentifier"]]

    def __init__(
        self,
        *,
        key: str,
        name: "LocalizedString",
        languages: typing.Optional[typing.List[str]] = None,
        distribution_channels: typing.Optional[
            typing.List["ChannelResourceIdentifier"]
        ] = None,
        supply_channels: typing.Optional[
            typing.List["ChannelResourceIdentifier"]
        ] = None
    ) -> None:
        self.key = key
        self.name = name
        self.languages = languages
        self.distribution_channels = distribution_channels
        self.supply_channels = supply_channels
        super().__init__()

    def __repr__(self) -> str:
        return (
            "StoreDraft(key=%r, name=%r, languages=%r, distribution_channels=%r, supply_channels=%r)"
            % (
                self.key,
                self.name,
                self.languages,
                self.distribution_channels,
                self.supply_channels,
            )
        )


class StoreKeyReference(KeyReference):
    def __init__(self, *, key: str) -> None:
        super().__init__(type_id=ReferenceTypeId.STORE, key=key)

    def __repr__(self) -> str:
        return "StoreKeyReference(type_id=%r, key=%r)" % (self.type_id, self.key)


class StorePagedQueryResponse(_BaseType):
    #: :class:`int`
    limit: int
    #: :class:`int`
    count: int
    #: Optional :class:`int`
    total: typing.Optional[int]
    #: :class:`int`
    offset: int
    #: List of :class:`commercetools.types.Store`
    results: typing.Sequence["Store"]

    def __init__(
        self,
        *,
        limit: int,
        count: int,
        offset: int,
        results: typing.Sequence["Store"],
        total: typing.Optional[int] = None
    ) -> None:
        self.limit = limit
        self.count = count
        self.total = total
        self.offset = offset
        self.results = results
        super().__init__()

    def __repr__(self) -> str:
        return (
            "StorePagedQueryResponse(limit=%r, count=%r, total=%r, offset=%r, results=%r)"
            % (self.limit, self.count, self.total, self.offset, self.results)
        )


class StoreReference(Reference):
    #: Optional :class:`commercetools.types.Store`
    obj: typing.Optional["Store"]

    def __init__(self, *, id: str, obj: typing.Optional["Store"] = None) -> None:
        self.obj = obj
        super().__init__(type_id=ReferenceTypeId.STORE, id=id)

    def __repr__(self) -> str:
        return "StoreReference(type_id=%r, id=%r, obj=%r)" % (
            self.type_id,
            self.id,
            self.obj,
        )


class StoreResourceIdentifier(ResourceIdentifier):
    def __init__(
        self, *, id: typing.Optional[str] = None, key: typing.Optional[str] = None
    ) -> None:
        super().__init__(type_id=ReferenceTypeId.STORE, id=id, key=key)

    def __repr__(self) -> str:
        return "StoreResourceIdentifier(type_id=%r, id=%r, key=%r)" % (
            self.type_id,
            self.id,
            self.key,
        )


class StoreUpdate(_BaseType):
    #: :class:`int`
    version: int
    #: :class:`list`
    actions: list

    def __init__(self, *, version: int, actions: list) -> None:
        self.version = version
        self.actions = actions
        super().__init__()

    def __repr__(self) -> str:
        return "StoreUpdate(version=%r, actions=%r)" % (self.version, self.actions)


class StoreUpdateAction(_BaseType):
    #: :class:`str`
    action: str

    def __init__(self, *, action: str) -> None:
        self.action = action
        super().__init__()

    def __repr__(self) -> str:
        return "StoreUpdateAction(action=%r)" % (self.action,)


class StoreSetLanguagesAction(StoreUpdateAction):
    #: Optional list of :class:`str`
    languages: typing.Optional[typing.List[str]]

    def __init__(self, *, languages: typing.Optional[typing.List[str]] = None) -> None:
        self.languages = languages
        super().__init__(action="setLanguages")

    def __repr__(self) -> str:
        return "StoreSetLanguagesAction(action=%r, languages=%r)" % (
            self.action,
            self.languages,
        )


class StoreSetNameAction(StoreUpdateAction):
    #: Optional :class:`commercetools.types.LocalizedString`
    name: typing.Optional["LocalizedString"]

    def __init__(self, *, name: typing.Optional["LocalizedString"] = None) -> None:
        self.name = name
        super().__init__(action="setName")

    def __repr__(self) -> str:
        return "StoreSetNameAction(action=%r, name=%r)" % (self.action, self.name)


class StoresAddDistributionChannelsAction(StoreUpdateAction):
    #: :class:`commercetools.types.ChannelResourceIdentifier` `(Named` ``distributionChannel`` `in Commercetools)`
    distribution_channel: "ChannelResourceIdentifier"

    def __init__(self, *, distribution_channel: "ChannelResourceIdentifier") -> None:
        self.distribution_channel = distribution_channel
        super().__init__(action="addDistributionChannel")

    def __repr__(self) -> str:
        return (
            "StoresAddDistributionChannelsAction(action=%r, distribution_channel=%r)"
            % (self.action, self.distribution_channel)
        )


class StoresAddSupplyChannelsAction(StoreUpdateAction):
    #: :class:`commercetools.types.ChannelResourceIdentifier` `(Named` ``supplyChannel`` `in Commercetools)`
    supply_channel: "ChannelResourceIdentifier"

    def __init__(self, *, supply_channel: "ChannelResourceIdentifier") -> None:
        self.supply_channel = supply_channel
        super().__init__(action="addSupplyChannel")

    def __repr__(self) -> str:
        return "StoresAddSupplyChannelsAction(action=%r, supply_channel=%r)" % (
            self.action,
            self.supply_channel,
        )


class StoresRemoveDistributionChannelsAction(StoreUpdateAction):
    #: :class:`commercetools.types.ChannelResourceIdentifier` `(Named` ``distributionChannel`` `in Commercetools)`
    distribution_channel: "ChannelResourceIdentifier"

    def __init__(self, *, distribution_channel: "ChannelResourceIdentifier") -> None:
        self.distribution_channel = distribution_channel
        super().__init__(action="removeDistributionChannel")

    def __repr__(self) -> str:
        return (
            "StoresRemoveDistributionChannelsAction(action=%r, distribution_channel=%r)"
            % (self.action, self.distribution_channel)
        )


class StoresRemoveSupplyChannelsAction(StoreUpdateAction):
    #: :class:`commercetools.types.ChannelResourceIdentifier` `(Named` ``supplyChannel`` `in Commercetools)`
    supply_channel: "ChannelResourceIdentifier"

    def __init__(self, *, supply_channel: "ChannelResourceIdentifier") -> None:
        self.supply_channel = supply_channel
        super().__init__(action="removeSupplyChannel")

    def __repr__(self) -> str:
        return "StoresRemoveSupplyChannelsAction(action=%r, supply_channel=%r)" % (
            self.action,
            self.supply_channel,
        )


class StoresSetDistributionChannelsAction(StoreUpdateAction):
    #: Optional list of :class:`commercetools.types.ChannelResourceIdentifier` `(Named` ``distributionChannels`` `in Commercetools)`
    distribution_channels: typing.Optional[typing.List["ChannelResourceIdentifier"]]

    def __init__(
        self,
        *,
        distribution_channels: typing.Optional[
            typing.List["ChannelResourceIdentifier"]
        ] = None
    ) -> None:
        self.distribution_channels = distribution_channels
        super().__init__(action="setDistributionChannels")

    def __repr__(self) -> str:
        return (
            "StoresSetDistributionChannelsAction(action=%r, distribution_channels=%r)"
            % (self.action, self.distribution_channels)
        )


class StoresSetSupplyChannelsAction(StoreUpdateAction):
    #: Optional list of :class:`commercetools.types.ChannelResourceIdentifier` `(Named` ``supplyChannels`` `in Commercetools)`
    supply_channels: typing.Optional[typing.List["ChannelResourceIdentifier"]]

    def __init__(
        self,
        *,
        supply_channels: typing.Optional[
            typing.List["ChannelResourceIdentifier"]
        ] = None
    ) -> None:
        self.supply_channels = supply_channels
        super().__init__(action="setSupplyChannels")

    def __repr__(self) -> str:
        return "StoresSetSupplyChannelsAction(action=%r, supply_channels=%r)" % (
            self.action,
            self.supply_channels,
        )



# DO NOT EDIT! This file is automatically generated
import datetime
import typing

from commercetools.types._abstract import _BaseType
from commercetools.types._common import (
    BaseResource,
    Reference,
    ReferenceTypeId,
    ResourceIdentifier,
)

if typing.TYPE_CHECKING:
    from ._common import CreatedBy, LastModifiedBy, LocalizedString
    from ._customer import CustomerReference, CustomerResourceIdentifier
    from ._product import ProductVariant
    from ._product_type import ProductTypeReference
    from ._type import (
        CustomFields,
        CustomFieldsDraft,
        FieldContainer,
        TypeResourceIdentifier,
    )
__all__ = [
    "MyShoppingList",
    "ShoppingList",
    "ShoppingListAddLineItemAction",
    "ShoppingListAddTextLineItemAction",
    "ShoppingListChangeLineItemQuantityAction",
    "ShoppingListChangeLineItemsOrderAction",
    "ShoppingListChangeNameAction",
    "ShoppingListChangeTextLineItemNameAction",
    "ShoppingListChangeTextLineItemQuantityAction",
    "ShoppingListChangeTextLineItemsOrderAction",
    "ShoppingListDraft",
    "ShoppingListLineItem",
    "ShoppingListLineItemDraft",
    "ShoppingListPagedQueryResponse",
    "ShoppingListReference",
    "ShoppingListRemoveLineItemAction",
    "ShoppingListRemoveTextLineItemAction",
    "ShoppingListResourceIdentifier",
    "ShoppingListSetAnonymousIdAction",
    "ShoppingListSetCustomFieldAction",
    "ShoppingListSetCustomTypeAction",
    "ShoppingListSetCustomerAction",
    "ShoppingListSetDeleteDaysAfterLastModificationAction",
    "ShoppingListSetDescriptionAction",
    "ShoppingListSetKeyAction",
    "ShoppingListSetLineItemCustomFieldAction",
    "ShoppingListSetLineItemCustomTypeAction",
    "ShoppingListSetSlugAction",
    "ShoppingListSetTextLineItemCustomFieldAction",
    "ShoppingListSetTextLineItemCustomTypeAction",
    "ShoppingListSetTextLineItemDescriptionAction",
    "ShoppingListUpdate",
    "ShoppingListUpdateAction",
    "TextLineItem",
    "TextLineItemDraft",
]


class MyShoppingList(BaseResource):
    #: :class:`str`
    id: str
    #: :class:`int`
    version: int
    #: :class:`datetime.datetime` `(Named` ``createdAt`` `in Commercetools)`
    created_at: datetime.datetime
    #: :class:`datetime.datetime` `(Named` ``lastModifiedAt`` `in Commercetools)`
    last_modified_at: datetime.datetime
    #: Optional :class:`commercetools.types.LastModifiedBy` `(Named` ``lastModifiedBy`` `in Commercetools)`
    last_modified_by: typing.Optional["LastModifiedBy"]
    #: Optional :class:`commercetools.types.CreatedBy` `(Named` ``createdBy`` `in Commercetools)`
    created_by: typing.Optional["CreatedBy"]
    #: Optional :class:`commercetools.types.CustomFields`
    custom: typing.Optional["CustomFields"]
    #: Optional :class:`commercetools.types.CustomerReference`
    customer: typing.Optional["CustomerReference"]
    #: Optional :class:`int` `(Named` ``deleteDaysAfterLastModification`` `in Commercetools)`
    delete_days_after_last_modification: typing.Optional[int]
    #: Optional :class:`commercetools.types.LocalizedString`
    description: typing.Optional["LocalizedString"]
    #: Optional :class:`str`
    key: typing.Optional[str]
    #: Optional list of :class:`commercetools.types.ShoppingListLineItem` `(Named` ``lineItems`` `in Commercetools)`
    line_items: typing.Optional[typing.List["ShoppingListLineItem"]]
    #: :class:`commercetools.types.LocalizedString`
    name: "LocalizedString"
    #: Optional :class:`commercetools.types.LocalizedString`
    slug: typing.Optional["LocalizedString"]
    #: Optional list of :class:`commercetools.types.TextLineItem` `(Named` ``textLineItems`` `in Commercetools)`
    text_line_items: typing.Optional[typing.List["TextLineItem"]]
    #: Optional :class:`str` `(Named` ``anonymousId`` `in Commercetools)`
    anonymous_id: typing.Optional[str]

    def __init__(
        self,
        *,
        id: str,
        version: int,
        created_at: datetime.datetime,
        last_modified_at: datetime.datetime,
        name: "LocalizedString",
        last_modified_by: typing.Optional["LastModifiedBy"] = None,
        created_by: typing.Optional["CreatedBy"] = None,
        custom: typing.Optional["CustomFields"] = None,
        customer: typing.Optional["CustomerReference"] = None,
        delete_days_after_last_modification: typing.Optional[int] = None,
        description: typing.Optional["LocalizedString"] = None,
        key: typing.Optional[str] = None,
        line_items: typing.Optional[typing.List["ShoppingListLineItem"]] = None,
        slug: typing.Optional["LocalizedString"] = None,
        text_line_items: typing.Optional[typing.List["TextLineItem"]] = None,
        anonymous_id: typing.Optional[str] = None
    ) -> None:
        self.id = id
        self.version = version
        self.created_at = created_at
        self.last_modified_at = last_modified_at
        self.last_modified_by = last_modified_by
        self.created_by = created_by
        self.custom = custom
        self.customer = customer
        self.delete_days_after_last_modification = delete_days_after_last_modification
        self.description = description
        self.key = key
        self.line_items = line_items
        self.name = name
        self.slug = slug
        self.text_line_items = text_line_items
        self.anonymous_id = anonymous_id
        super().__init__(
            id=id,
            version=version,
            created_at=created_at,
            last_modified_at=last_modified_at,
        )

    def __repr__(self) -> str:
        return (
            "MyShoppingList(id=%r, version=%r, created_at=%r, last_modified_at=%r, last_modified_by=%r, created_by=%r, custom=%r, customer=%r, delete_days_after_last_modification=%r, description=%r, key=%r, line_items=%r, name=%r, slug=%r, text_line_items=%r, anonymous_id=%r)"
            % (
                self.id,
                self.version,
                self.created_at,
                self.last_modified_at,
                self.last_modified_by,
                self.created_by,
                self.custom,
                self.customer,
                self.delete_days_after_last_modification,
                self.description,
                self.key,
                self.line_items,
                self.name,
                self.slug,
                self.text_line_items,
                self.anonymous_id,
            )
        )


class ShoppingList(BaseResource):
    #: :class:`str`
    id: str
    #: :class:`int`
    version: int
    #: :class:`datetime.datetime` `(Named` ``createdAt`` `in Commercetools)`
    created_at: datetime.datetime
    #: :class:`datetime.datetime` `(Named` ``lastModifiedAt`` `in Commercetools)`
    last_modified_at: datetime.datetime
    #: Optional :class:`commercetools.types.LastModifiedBy` `(Named` ``lastModifiedBy`` `in Commercetools)`
    last_modified_by: typing.Optional["LastModifiedBy"]
    #: Optional :class:`commercetools.types.CreatedBy` `(Named` ``createdBy`` `in Commercetools)`
    created_by: typing.Optional["CreatedBy"]
    #: Optional :class:`commercetools.types.CustomFields`
    custom: typing.Optional["CustomFields"]
    #: Optional :class:`commercetools.types.CustomerReference`
    customer: typing.Optional["CustomerReference"]
    #: Optional :class:`int` `(Named` ``deleteDaysAfterLastModification`` `in Commercetools)`
    delete_days_after_last_modification: typing.Optional[int]
    #: Optional :class:`commercetools.types.LocalizedString`
    description: typing.Optional["LocalizedString"]
    #: Optional :class:`str`
    key: typing.Optional[str]
    #: Optional list of :class:`commercetools.types.ShoppingListLineItem` `(Named` ``lineItems`` `in Commercetools)`
    line_items: typing.Optional[typing.List["ShoppingListLineItem"]]
    #: :class:`commercetools.types.LocalizedString`
    name: "LocalizedString"
    #: Optional :class:`commercetools.types.LocalizedString`
    slug: typing.Optional["LocalizedString"]
    #: Optional list of :class:`commercetools.types.TextLineItem` `(Named` ``textLineItems`` `in Commercetools)`
    text_line_items: typing.Optional[typing.List["TextLineItem"]]
    #: Optional :class:`str` `(Named` ``anonymousId`` `in Commercetools)`
    anonymous_id: typing.Optional[str]

    def __init__(
        self,
        *,
        id: str,
        version: int,
        created_at: datetime.datetime,
        last_modified_at: datetime.datetime,
        name: "LocalizedString",
        last_modified_by: typing.Optional["LastModifiedBy"] = None,
        created_by: typing.Optional["CreatedBy"] = None,
        custom: typing.Optional["CustomFields"] = None,
        customer: typing.Optional["CustomerReference"] = None,
        delete_days_after_last_modification: typing.Optional[int] = None,
        description: typing.Optional["LocalizedString"] = None,
        key: typing.Optional[str] = None,
        line_items: typing.Optional[typing.List["ShoppingListLineItem"]] = None,
        slug: typing.Optional["LocalizedString"] = None,
        text_line_items: typing.Optional[typing.List["TextLineItem"]] = None,
        anonymous_id: typing.Optional[str] = None
    ) -> None:
        self.id = id
        self.version = version
        self.created_at = created_at
        self.last_modified_at = last_modified_at
        self.last_modified_by = last_modified_by
        self.created_by = created_by
        self.custom = custom
        self.customer = customer
        self.delete_days_after_last_modification = delete_days_after_last_modification
        self.description = description
        self.key = key
        self.line_items = line_items
        self.name = name
        self.slug = slug
        self.text_line_items = text_line_items
        self.anonymous_id = anonymous_id
        super().__init__(
            id=id,
            version=version,
            created_at=created_at,
            last_modified_at=last_modified_at,
        )

    def __repr__(self) -> str:
        return (
            "ShoppingList(id=%r, version=%r, created_at=%r, last_modified_at=%r, last_modified_by=%r, created_by=%r, custom=%r, customer=%r, delete_days_after_last_modification=%r, description=%r, key=%r, line_items=%r, name=%r, slug=%r, text_line_items=%r, anonymous_id=%r)"
            % (
                self.id,
                self.version,
                self.created_at,
                self.last_modified_at,
                self.last_modified_by,
                self.created_by,
                self.custom,
                self.customer,
                self.delete_days_after_last_modification,
                self.description,
                self.key,
                self.line_items,
                self.name,
                self.slug,
                self.text_line_items,
                self.anonymous_id,
            )
        )


class ShoppingListDraft(_BaseType):
    #: Optional :class:`commercetools.types.CustomFieldsDraft`
    custom: typing.Optional["CustomFieldsDraft"]
    #: Optional :class:`commercetools.types.CustomerResourceIdentifier`
    customer: typing.Optional["CustomerResourceIdentifier"]
    #: Optional :class:`int` `(Named` ``deleteDaysAfterLastModification`` `in Commercetools)`
    delete_days_after_last_modification: typing.Optional[int]
    #: Optional :class:`commercetools.types.LocalizedString`
    description: typing.Optional["LocalizedString"]
    #: Optional :class:`str`
    key: typing.Optional[str]
    #: Optional list of :class:`commercetools.types.ShoppingListLineItemDraft` `(Named` ``lineItems`` `in Commercetools)`
    line_items: typing.Optional[typing.List["ShoppingListLineItemDraft"]]
    #: :class:`commercetools.types.LocalizedString`
    name: "LocalizedString"
    #: Optional :class:`commercetools.types.LocalizedString`
    slug: typing.Optional["LocalizedString"]
    #: Optional list of :class:`commercetools.types.TextLineItemDraft` `(Named` ``textLineItems`` `in Commercetools)`
    text_line_items: typing.Optional[typing.List["TextLineItemDraft"]]
    #: Optional :class:`str` `(Named` ``anonymousId`` `in Commercetools)`
    anonymous_id: typing.Optional[str]

    def __init__(
        self,
        *,
        name: "LocalizedString",
        custom: typing.Optional["CustomFieldsDraft"] = None,
        customer: typing.Optional["CustomerResourceIdentifier"] = None,
        delete_days_after_last_modification: typing.Optional[int] = None,
        description: typing.Optional["LocalizedString"] = None,
        key: typing.Optional[str] = None,
        line_items: typing.Optional[typing.List["ShoppingListLineItemDraft"]] = None,
        slug: typing.Optional["LocalizedString"] = None,
        text_line_items: typing.Optional[typing.List["TextLineItemDraft"]] = None,
        anonymous_id: typing.Optional[str] = None
    ) -> None:
        self.custom = custom
        self.customer = customer
        self.delete_days_after_last_modification = delete_days_after_last_modification
        self.description = description
        self.key = key
        self.line_items = line_items
        self.name = name
        self.slug = slug
        self.text_line_items = text_line_items
        self.anonymous_id = anonymous_id
        super().__init__()

    def __repr__(self) -> str:
        return (
            "ShoppingListDraft(custom=%r, customer=%r, delete_days_after_last_modification=%r, description=%r, key=%r, line_items=%r, name=%r, slug=%r, text_line_items=%r, anonymous_id=%r)"
            % (
                self.custom,
                self.customer,
                self.delete_days_after_last_modification,
                self.description,
                self.key,
                self.line_items,
                self.name,
                self.slug,
                self.text_line_items,
                self.anonymous_id,
            )
        )


class ShoppingListLineItem(_BaseType):
    #: :class:`datetime.datetime` `(Named` ``addedAt`` `in Commercetools)`
    added_at: datetime.datetime
    #: Optional :class:`commercetools.types.CustomFields`
    custom: typing.Optional["CustomFields"]
    #: Optional :class:`datetime.datetime` `(Named` ``deactivatedAt`` `in Commercetools)`
    deactivated_at: typing.Optional[datetime.datetime]
    #: :class:`str`
    id: str
    #: :class:`commercetools.types.LocalizedString`
    name: "LocalizedString"
    #: :class:`str` `(Named` ``productId`` `in Commercetools)`
    product_id: str
    #: Optional :class:`commercetools.types.LocalizedString` `(Named` ``productSlug`` `in Commercetools)`
    product_slug: typing.Optional["LocalizedString"]
    #: :class:`commercetools.types.ProductTypeReference` `(Named` ``productType`` `in Commercetools)`
    product_type: "ProductTypeReference"
    #: :class:`int`
    quantity: int
    #: Optional :class:`commercetools.types.ProductVariant`
    variant: typing.Optional["ProductVariant"]
    #: Optional :class:`int` `(Named` ``variantId`` `in Commercetools)`
    variant_id: typing.Optional[int]

    def __init__(
        self,
        *,
        added_at: datetime.datetime,
        id: str,
        name: "LocalizedString",
        product_id: str,
        product_type: "ProductTypeReference",
        quantity: int,
        custom: typing.Optional["CustomFields"] = None,
        deactivated_at: typing.Optional[datetime.datetime] = None,
        product_slug: typing.Optional["LocalizedString"] = None,
        variant: typing.Optional["ProductVariant"] = None,
        variant_id: typing.Optional[int] = None
    ) -> None:
        self.added_at = added_at
        self.custom = custom
        self.deactivated_at = deactivated_at
        self.id = id
        self.name = name
        self.product_id = product_id
        self.product_slug = product_slug
        self.product_type = product_type
        self.quantity = quantity
        self.variant = variant
        self.variant_id = variant_id
        super().__init__()

    def __repr__(self) -> str:
        return (
            "ShoppingListLineItem(added_at=%r, custom=%r, deactivated_at=%r, id=%r, name=%r, product_id=%r, product_slug=%r, product_type=%r, quantity=%r, variant=%r, variant_id=%r)"
            % (
                self.added_at,
                self.custom,
                self.deactivated_at,
                self.id,
                self.name,
                self.product_id,
                self.product_slug,
                self.product_type,
                self.quantity,
                self.variant,
                self.variant_id,
            )
        )


class ShoppingListLineItemDraft(_BaseType):
    #: Optional :class:`datetime.datetime` `(Named` ``addedAt`` `in Commercetools)`
    added_at: typing.Optional[datetime.datetime]
    #: Optional :class:`commercetools.types.CustomFieldsDraft`
    custom: typing.Optional["CustomFieldsDraft"]
    #: Optional :class:`str`
    sku: typing.Optional[str]
    #: Optional :class:`str` `(Named` ``productId`` `in Commercetools)`
    product_id: typing.Optional[str]
    #: Optional :class:`int`
    quantity: typing.Optional[int]
    #: Optional :class:`int` `(Named` ``variantId`` `in Commercetools)`
    variant_id: typing.Optional[int]

    def __init__(
        self,
        *,
        added_at: typing.Optional[datetime.datetime] = None,
        custom: typing.Optional["CustomFieldsDraft"] = None,
        sku: typing.Optional[str] = None,
        product_id: typing.Optional[str] = None,
        quantity: typing.Optional[int] = None,
        variant_id: typing.Optional[int] = None
    ) -> None:
        self.added_at = added_at
        self.custom = custom
        self.sku = sku
        self.product_id = product_id
        self.quantity = quantity
        self.variant_id = variant_id
        super().__init__()

    def __repr__(self) -> str:
        return (
            "ShoppingListLineItemDraft(added_at=%r, custom=%r, sku=%r, product_id=%r, quantity=%r, variant_id=%r)"
            % (
                self.added_at,
                self.custom,
                self.sku,
                self.product_id,
                self.quantity,
                self.variant_id,
            )
        )


class ShoppingListPagedQueryResponse(_BaseType):
    #: :class:`int`
    limit: int
    #: :class:`int`
    count: int
    #: Optional :class:`int`
    total: typing.Optional[int]
    #: :class:`int`
    offset: int
    #: List of :class:`commercetools.types.ShoppingList`
    results: typing.Sequence["ShoppingList"]

    def __init__(
        self,
        *,
        limit: int,
        count: int,
        offset: int,
        results: typing.Sequence["ShoppingList"],
        total: typing.Optional[int] = None
    ) -> None:
        self.limit = limit
        self.count = count
        self.total = total
        self.offset = offset
        self.results = results
        super().__init__()

    def __repr__(self) -> str:
        return (
            "ShoppingListPagedQueryResponse(limit=%r, count=%r, total=%r, offset=%r, results=%r)"
            % (self.limit, self.count, self.total, self.offset, self.results)
        )


class ShoppingListReference(Reference):
    #: Optional :class:`commercetools.types.ShoppingList`
    obj: typing.Optional["ShoppingList"]

    def __init__(self, *, id: str, obj: typing.Optional["ShoppingList"] = None) -> None:
        self.obj = obj
        super().__init__(type_id=ReferenceTypeId.SHOPPING_LIST, id=id)

    def __repr__(self) -> str:
        return "ShoppingListReference(type_id=%r, id=%r, obj=%r)" % (
            self.type_id,
            self.id,
            self.obj,
        )


class ShoppingListResourceIdentifier(ResourceIdentifier):
    def __init__(
        self, *, id: typing.Optional[str] = None, key: typing.Optional[str] = None
    ) -> None:
        super().__init__(type_id=ReferenceTypeId.SHOPPING_LIST, id=id, key=key)

    def __repr__(self) -> str:
        return "ShoppingListResourceIdentifier(type_id=%r, id=%r, key=%r)" % (
            self.type_id,
            self.id,
            self.key,
        )


class ShoppingListUpdate(_BaseType):
    #: :class:`int`
    version: int
    #: :class:`list`
    actions: list

    def __init__(self, *, version: int, actions: list) -> None:
        self.version = version
        self.actions = actions
        super().__init__()

    def __repr__(self) -> str:
        return "ShoppingListUpdate(version=%r, actions=%r)" % (
            self.version,
            self.actions,
        )


class ShoppingListUpdateAction(_BaseType):
    #: :class:`str`
    action: str

    def __init__(self, *, action: str) -> None:
        self.action = action
        super().__init__()

    def __repr__(self) -> str:
        return "ShoppingListUpdateAction(action=%r)" % (self.action,)


class TextLineItem(_BaseType):
    #: :class:`datetime.datetime` `(Named` ``addedAt`` `in Commercetools)`
    added_at: datetime.datetime
    #: Optional :class:`commercetools.types.CustomFields`
    custom: typing.Optional["CustomFields"]
    #: Optional :class:`commercetools.types.LocalizedString`
    description: typing.Optional["LocalizedString"]
    #: :class:`str`
    id: str
    #: :class:`commercetools.types.LocalizedString`
    name: "LocalizedString"
    #: :class:`int`
    quantity: int

    def __init__(
        self,
        *,
        added_at: datetime.datetime,
        id: str,
        name: "LocalizedString",
        quantity: int,
        custom: typing.Optional["CustomFields"] = None,
        description: typing.Optional["LocalizedString"] = None
    ) -> None:
        self.added_at = added_at
        self.custom = custom
        self.description = description
        self.id = id
        self.name = name
        self.quantity = quantity
        super().__init__()

    def __repr__(self) -> str:
        return (
            "TextLineItem(added_at=%r, custom=%r, description=%r, id=%r, name=%r, quantity=%r)"
            % (
                self.added_at,
                self.custom,
                self.description,
                self.id,
                self.name,
                self.quantity,
            )
        )


class TextLineItemDraft(_BaseType):
    #: Optional :class:`datetime.datetime` `(Named` ``addedAt`` `in Commercetools)`
    added_at: typing.Optional[datetime.datetime]
    #: Optional :class:`commercetools.types.CustomFieldsDraft`
    custom: typing.Optional["CustomFieldsDraft"]
    #: Optional :class:`commercetools.types.LocalizedString`
    description: typing.Optional["LocalizedString"]
    #: :class:`commercetools.types.LocalizedString`
    name: "LocalizedString"
    #: Optional :class:`int`
    quantity: typing.Optional[int]

    def __init__(
        self,
        *,
        name: "LocalizedString",
        added_at: typing.Optional[datetime.datetime] = None,
        custom: typing.Optional["CustomFieldsDraft"] = None,
        description: typing.Optional["LocalizedString"] = None,
        quantity: typing.Optional[int] = None
    ) -> None:
        self.added_at = added_at
        self.custom = custom
        self.description = description
        self.name = name
        self.quantity = quantity
        super().__init__()

    def __repr__(self) -> str:
        return (
            "TextLineItemDraft(added_at=%r, custom=%r, description=%r, name=%r, quantity=%r)"
            % (self.added_at, self.custom, self.description, self.name, self.quantity)
        )


class ShoppingListAddLineItemAction(ShoppingListUpdateAction):
    #: Optional :class:`str`
    sku: typing.Optional[str]
    #: Optional :class:`str` `(Named` ``productId`` `in Commercetools)`
    product_id: typing.Optional[str]
    #: Optional :class:`int` `(Named` ``variantId`` `in Commercetools)`
    variant_id: typing.Optional[int]
    #: Optional :class:`int`
    quantity: typing.Optional[int]
    #: Optional :class:`datetime.datetime` `(Named` ``addedAt`` `in Commercetools)`
    added_at: typing.Optional[datetime.datetime]
    #: Optional :class:`commercetools.types.CustomFieldsDraft`
    custom: typing.Optional["CustomFieldsDraft"]

    def __init__(
        self,
        *,
        sku: typing.Optional[str] = None,
        product_id: typing.Optional[str] = None,
        variant_id: typing.Optional[int] = None,
        quantity: typing.Optional[int] = None,
        added_at: typing.Optional[datetime.datetime] = None,
        custom: typing.Optional["CustomFieldsDraft"] = None
    ) -> None:
        self.sku = sku
        self.product_id = product_id
        self.variant_id = variant_id
        self.quantity = quantity
        self.added_at = added_at
        self.custom = custom
        super().__init__(action="addLineItem")

    def __repr__(self) -> str:
        return (
            "ShoppingListAddLineItemAction(action=%r, sku=%r, product_id=%r, variant_id=%r, quantity=%r, added_at=%r, custom=%r)"
            % (
                self.action,
                self.sku,
                self.product_id,
                self.variant_id,
                self.quantity,
                self.added_at,
                self.custom,
            )
        )


class ShoppingListAddTextLineItemAction(ShoppingListUpdateAction):
    #: :class:`commercetools.types.LocalizedString`
    name: "LocalizedString"
    #: Optional :class:`commercetools.types.LocalizedString`
    description: typing.Optional["LocalizedString"]
    #: Optional :class:`int`
    quantity: typing.Optional[int]
    #: Optional :class:`datetime.datetime` `(Named` ``addedAt`` `in Commercetools)`
    added_at: typing.Optional[datetime.datetime]
    #: Optional :class:`commercetools.types.CustomFieldsDraft`
    custom: typing.Optional["CustomFieldsDraft"]

    def __init__(
        self,
        *,
        name: "LocalizedString",
        description: typing.Optional["LocalizedString"] = None,
        quantity: typing.Optional[int] = None,
        added_at: typing.Optional[datetime.datetime] = None,
        custom: typing.Optional["CustomFieldsDraft"] = None
    ) -> None:
        self.name = name
        self.description = description
        self.quantity = quantity
        self.added_at = added_at
        self.custom = custom
        super().__init__(action="addTextLineItem")

    def __repr__(self) -> str:
        return (
            "ShoppingListAddTextLineItemAction(action=%r, name=%r, description=%r, quantity=%r, added_at=%r, custom=%r)"
            % (
                self.action,
                self.name,
                self.description,
                self.quantity,
                self.added_at,
                self.custom,
            )
        )


class ShoppingListChangeLineItemQuantityAction(ShoppingListUpdateAction):
    #: :class:`str` `(Named` ``lineItemId`` `in Commercetools)`
    line_item_id: str
    #: :class:`int`
    quantity: int

    def __init__(self, *, line_item_id: str, quantity: int) -> None:
        self.line_item_id = line_item_id
        self.quantity = quantity
        super().__init__(action="changeLineItemQuantity")

    def __repr__(self) -> str:
        return (
            "ShoppingListChangeLineItemQuantityAction(action=%r, line_item_id=%r, quantity=%r)"
            % (self.action, self.line_item_id, self.quantity)
        )


class ShoppingListChangeLineItemsOrderAction(ShoppingListUpdateAction):
    #: List of :class:`str` `(Named` ``lineItemOrder`` `in Commercetools)`
    line_item_order: typing.List[str]

    def __init__(self, *, line_item_order: typing.List[str]) -> None:
        self.line_item_order = line_item_order
        super().__init__(action="changeLineItemsOrder")

    def __repr__(self) -> str:
        return (
            "ShoppingListChangeLineItemsOrderAction(action=%r, line_item_order=%r)"
            % (self.action, self.line_item_order)
        )


class ShoppingListChangeNameAction(ShoppingListUpdateAction):
    #: :class:`commercetools.types.LocalizedString`
    name: "LocalizedString"

    def __init__(self, *, name: "LocalizedString") -> None:
        self.name = name
        super().__init__(action="changeName")

    def __repr__(self) -> str:
        return "ShoppingListChangeNameAction(action=%r, name=%r)" % (
            self.action,
            self.name,
        )


class ShoppingListChangeTextLineItemNameAction(ShoppingListUpdateAction):
    #: :class:`str` `(Named` ``textLineItemId`` `in Commercetools)`
    text_line_item_id: str
    #: :class:`commercetools.types.LocalizedString`
    name: "LocalizedString"

    def __init__(self, *, text_line_item_id: str, name: "LocalizedString") -> None:
        self.text_line_item_id = text_line_item_id
        self.name = name
        super().__init__(action="changeTextLineItemName")

    def __repr__(self) -> str:
        return (
            "ShoppingListChangeTextLineItemNameAction(action=%r, text_line_item_id=%r, name=%r)"
            % (self.action, self.text_line_item_id, self.name)
        )


class ShoppingListChangeTextLineItemQuantityAction(ShoppingListUpdateAction):
    #: :class:`str` `(Named` ``textLineItemId`` `in Commercetools)`
    text_line_item_id: str
    #: :class:`int`
    quantity: int

    def __init__(self, *, text_line_item_id: str, quantity: int) -> None:
        self.text_line_item_id = text_line_item_id
        self.quantity = quantity
        super().__init__(action="changeTextLineItemQuantity")

    def __repr__(self) -> str:
        return (
            "ShoppingListChangeTextLineItemQuantityAction(action=%r, text_line_item_id=%r, quantity=%r)"
            % (self.action, self.text_line_item_id, self.quantity)
        )


class ShoppingListChangeTextLineItemsOrderAction(ShoppingListUpdateAction):
    #: List of :class:`str` `(Named` ``textLineItemOrder`` `in Commercetools)`
    text_line_item_order: typing.List[str]

    def __init__(self, *, text_line_item_order: typing.List[str]) -> None:
        self.text_line_item_order = text_line_item_order
        super().__init__(action="changeTextLineItemsOrder")

    def __repr__(self) -> str:
        return (
            "ShoppingListChangeTextLineItemsOrderAction(action=%r, text_line_item_order=%r)"
            % (self.action, self.text_line_item_order)
        )


class ShoppingListRemoveLineItemAction(ShoppingListUpdateAction):
    #: :class:`str` `(Named` ``lineItemId`` `in Commercetools)`
    line_item_id: str
    #: Optional :class:`int`
    quantity: typing.Optional[int]

    def __init__(
        self, *, line_item_id: str, quantity: typing.Optional[int] = None
    ) -> None:
        self.line_item_id = line_item_id
        self.quantity = quantity
        super().__init__(action="removeLineItem")

    def __repr__(self) -> str:
        return (
            "ShoppingListRemoveLineItemAction(action=%r, line_item_id=%r, quantity=%r)"
            % (self.action, self.line_item_id, self.quantity)
        )


class ShoppingListRemoveTextLineItemAction(ShoppingListUpdateAction):
    #: :class:`str` `(Named` ``textLineItemId`` `in Commercetools)`
    text_line_item_id: str
    #: Optional :class:`int`
    quantity: typing.Optional[int]

    def __init__(
        self, *, text_line_item_id: str, quantity: typing.Optional[int] = None
    ) -> None:
        self.text_line_item_id = text_line_item_id
        self.quantity = quantity
        super().__init__(action="removeTextLineItem")

    def __repr__(self) -> str:
        return (
            "ShoppingListRemoveTextLineItemAction(action=%r, text_line_item_id=%r, quantity=%r)"
            % (self.action, self.text_line_item_id, self.quantity)
        )


class ShoppingListSetAnonymousIdAction(ShoppingListUpdateAction):
    #: Optional :class:`str` `(Named` ``anonymousId`` `in Commercetools)`
    anonymous_id: typing.Optional[str]

    def __init__(self, *, anonymous_id: typing.Optional[str] = None) -> None:
        self.anonymous_id = anonymous_id
        super().__init__(action="setAnonymousId")

    def __repr__(self) -> str:
        return "ShoppingListSetAnonymousIdAction(action=%r, anonymous_id=%r)" % (
            self.action,
            self.anonymous_id,
        )


class ShoppingListSetCustomFieldAction(ShoppingListUpdateAction):
    #: :class:`str`
    name: str
    #: Optional :class:`typing.Any`
    value: typing.Optional[typing.Any]

    def __init__(self, *, name: str, value: typing.Optional[typing.Any] = None) -> None:
        self.name = name
        self.value = value
        super().__init__(action="setCustomField")

    def __repr__(self) -> str:
        return "ShoppingListSetCustomFieldAction(action=%r, name=%r, value=%r)" % (
            self.action,
            self.name,
            self.value,
        )


class ShoppingListSetCustomTypeAction(ShoppingListUpdateAction):
    #: Optional :class:`commercetools.types.TypeResourceIdentifier`
    type: typing.Optional["TypeResourceIdentifier"]
    #: Optional :class:`commercetools.types.FieldContainer`
    fields: typing.Optional["FieldContainer"]

    def __init__(
        self,
        *,
        type: typing.Optional["TypeResourceIdentifier"] = None,
        fields: typing.Optional["FieldContainer"] = None
    ) -> None:
        self.type = type
        self.fields = fields
        super().__init__(action="setCustomType")

    def __repr__(self) -> str:
        return "ShoppingListSetCustomTypeAction(action=%r, type=%r, fields=%r)" % (
            self.action,
            self.type,
            self.fields,
        )


class ShoppingListSetCustomerAction(ShoppingListUpdateAction):
    #: Optional :class:`commercetools.types.CustomerResourceIdentifier`
    customer: typing.Optional["CustomerResourceIdentifier"]

    def __init__(
        self, *, customer: typing.Optional["CustomerResourceIdentifier"] = None
    ) -> None:
        self.customer = customer
        super().__init__(action="setCustomer")

    def __repr__(self) -> str:
        return "ShoppingListSetCustomerAction(action=%r, customer=%r)" % (
            self.action,
            self.customer,
        )


class ShoppingListSetDeleteDaysAfterLastModificationAction(ShoppingListUpdateAction):
    #: Optional :class:`int` `(Named` ``deleteDaysAfterLastModification`` `in Commercetools)`
    delete_days_after_last_modification: typing.Optional[int]

    def __init__(
        self, *, delete_days_after_last_modification: typing.Optional[int] = None
    ) -> None:
        self.delete_days_after_last_modification = delete_days_after_last_modification
        super().__init__(action="setDeleteDaysAfterLastModification")

    def __repr__(self) -> str:
        return (
            "ShoppingListSetDeleteDaysAfterLastModificationAction(action=%r, delete_days_after_last_modification=%r)"
            % (self.action, self.delete_days_after_last_modification)
        )


class ShoppingListSetDescriptionAction(ShoppingListUpdateAction):
    #: Optional :class:`commercetools.types.LocalizedString`
    description: typing.Optional["LocalizedString"]

    def __init__(
        self, *, description: typing.Optional["LocalizedString"] = None
    ) -> None:
        self.description = description
        super().__init__(action="setDescription")

    def __repr__(self) -> str:
        return "ShoppingListSetDescriptionAction(action=%r, description=%r)" % (
            self.action,
            self.description,
        )


class ShoppingListSetKeyAction(ShoppingListUpdateAction):
    #: Optional :class:`str`
    key: typing.Optional[str]

    def __init__(self, *, key: typing.Optional[str] = None) -> None:
        self.key = key
        super().__init__(action="setKey")

    def __repr__(self) -> str:
        return "ShoppingListSetKeyAction(action=%r, key=%r)" % (self.action, self.key)


class ShoppingListSetLineItemCustomFieldAction(ShoppingListUpdateAction):
    #: :class:`str` `(Named` ``lineItemId`` `in Commercetools)`
    line_item_id: str
    #: :class:`str`
    name: str
    #: Optional :class:`typing.Any`
    value: typing.Optional[typing.Any]

    def __init__(
        self, *, line_item_id: str, name: str, value: typing.Optional[typing.Any] = None
    ) -> None:
        self.line_item_id = line_item_id
        self.name = name
        self.value = value
        super().__init__(action="setLineItemCustomField")

    def __repr__(self) -> str:
        return (
            "ShoppingListSetLineItemCustomFieldAction(action=%r, line_item_id=%r, name=%r, value=%r)"
            % (self.action, self.line_item_id, self.name, self.value)
        )


class ShoppingListSetLineItemCustomTypeAction(ShoppingListUpdateAction):
    #: :class:`str` `(Named` ``lineItemId`` `in Commercetools)`
    line_item_id: str
    #: Optional :class:`commercetools.types.TypeResourceIdentifier`
    type: typing.Optional["TypeResourceIdentifier"]
    #: Optional :class:`commercetools.types.FieldContainer`
    fields: typing.Optional["FieldContainer"]

    def __init__(
        self,
        *,
        line_item_id: str,
        type: typing.Optional["TypeResourceIdentifier"] = None,
        fields: typing.Optional["FieldContainer"] = None
    ) -> None:
        self.line_item_id = line_item_id
        self.type = type
        self.fields = fields
        super().__init__(action="setLineItemCustomType")

    def __repr__(self) -> str:
        return (
            "ShoppingListSetLineItemCustomTypeAction(action=%r, line_item_id=%r, type=%r, fields=%r)"
            % (self.action, self.line_item_id, self.type, self.fields)
        )


class ShoppingListSetSlugAction(ShoppingListUpdateAction):
    #: Optional :class:`commercetools.types.LocalizedString`
    slug: typing.Optional["LocalizedString"]

    def __init__(self, *, slug: typing.Optional["LocalizedString"] = None) -> None:
        self.slug = slug
        super().__init__(action="setSlug")

    def __repr__(self) -> str:
        return "ShoppingListSetSlugAction(action=%r, slug=%r)" % (
            self.action,
            self.slug,
        )


class ShoppingListSetTextLineItemCustomFieldAction(ShoppingListUpdateAction):
    #: :class:`str` `(Named` ``textLineItemId`` `in Commercetools)`
    text_line_item_id: str
    #: :class:`str`
    name: str
    #: Optional :class:`typing.Any`
    value: typing.Optional[typing.Any]

    def __init__(
        self,
        *,
        text_line_item_id: str,
        name: str,
        value: typing.Optional[typing.Any] = None
    ) -> None:
        self.text_line_item_id = text_line_item_id
        self.name = name
        self.value = value
        super().__init__(action="setTextLineItemCustomField")

    def __repr__(self) -> str:
        return (
            "ShoppingListSetTextLineItemCustomFieldAction(action=%r, text_line_item_id=%r, name=%r, value=%r)"
            % (self.action, self.text_line_item_id, self.name, self.value)
        )


class ShoppingListSetTextLineItemCustomTypeAction(ShoppingListUpdateAction):
    #: :class:`str` `(Named` ``textLineItemId`` `in Commercetools)`
    text_line_item_id: str
    #: Optional :class:`commercetools.types.TypeResourceIdentifier`
    type: typing.Optional["TypeResourceIdentifier"]
    #: Optional :class:`commercetools.types.FieldContainer`
    fields: typing.Optional["FieldContainer"]

    def __init__(
        self,
        *,
        text_line_item_id: str,
        type: typing.Optional["TypeResourceIdentifier"] = None,
        fields: typing.Optional["FieldContainer"] = None
    ) -> None:
        self.text_line_item_id = text_line_item_id
        self.type = type
        self.fields = fields
        super().__init__(action="setTextLineItemCustomType")

    def __repr__(self) -> str:
        return (
            "ShoppingListSetTextLineItemCustomTypeAction(action=%r, text_line_item_id=%r, type=%r, fields=%r)"
            % (self.action, self.text_line_item_id, self.type, self.fields)
        )


class ShoppingListSetTextLineItemDescriptionAction(ShoppingListUpdateAction):
    #: :class:`str` `(Named` ``textLineItemId`` `in Commercetools)`
    text_line_item_id: str
    #: Optional :class:`commercetools.types.LocalizedString`
    description: typing.Optional["LocalizedString"]

    def __init__(
        self,
        *,
        text_line_item_id: str,
        description: typing.Optional["LocalizedString"] = None
    ) -> None:
        self.text_line_item_id = text_line_item_id
        self.description = description
        super().__init__(action="setTextLineItemDescription")

    def __repr__(self) -> str:
        return (
            "ShoppingListSetTextLineItemDescriptionAction(action=%r, text_line_item_id=%r, description=%r)"
            % (self.action, self.text_line_item_id, self.description)
        )



# DO NOT EDIT! This file is automatically generated
import marshmallow
import marshmallow_enum

from commercetools import helpers, types
from commercetools._schemas._common import BaseResourceSchema

__all__ = [
    "ExtensionAWSLambdaDestinationSchema",
    "ExtensionAuthorizationHeaderAuthenticationSchema",
    "ExtensionAzureFunctionsAuthenticationSchema",
    "ExtensionChangeDestinationActionSchema",
    "ExtensionChangeTriggersActionSchema",
    "ExtensionDestinationSchema",
    "ExtensionDraftSchema",
    "ExtensionHttpDestinationAuthenticationSchema",
    "ExtensionHttpDestinationSchema",
    "ExtensionInputSchema",
    "ExtensionPagedQueryResponseSchema",
    "ExtensionSchema",
    "ExtensionSetKeyActionSchema",
    "ExtensionSetTimeoutInMsActionSchema",
    "ExtensionTriggerSchema",
    "ExtensionUpdateActionSchema",
    "ExtensionUpdateSchema",
]


class ExtensionDestinationSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ExtensionDestination`."""

    type = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.ExtensionDestination(**data)


class ExtensionDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ExtensionDraft`."""

    key = marshmallow.fields.String(allow_none=True, missing=None)
    destination = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "AWSLambda": "commercetools._schemas._extension.ExtensionAWSLambdaDestinationSchema",
            "HTTP": "commercetools._schemas._extension.ExtensionHttpDestinationSchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    triggers = helpers.LazyNestedField(
        nested="commercetools._schemas._extension.ExtensionTriggerSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )
    timeout_in_ms = marshmallow.fields.Integer(
        allow_none=True, missing=None, data_key="timeoutInMs"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ExtensionDraft(**data)


class ExtensionHttpDestinationAuthenticationSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ExtensionHttpDestinationAuthentication`."""

    type = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.ExtensionHttpDestinationAuthentication(**data)


class ExtensionInputSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ExtensionInput`."""

    action = marshmallow_enum.EnumField(types.ExtensionAction, by_value=True)
    resource = helpers.Discriminator(
        discriminator_field=("typeId", "type_id"),
        discriminator_schemas={
            "cart-discount": "commercetools._schemas._cart_discount.CartDiscountReferenceSchema",
            "cart": "commercetools._schemas._cart.CartReferenceSchema",
            "category": "commercetools._schemas._category.CategoryReferenceSchema",
            "channel": "commercetools._schemas._channel.ChannelReferenceSchema",
            "key-value-document": "commercetools._schemas._custom_object.CustomObjectReferenceSchema",
            "customer-group": "commercetools._schemas._customer_group.CustomerGroupReferenceSchema",
            "customer": "commercetools._schemas._customer.CustomerReferenceSchema",
            "discount-code": "commercetools._schemas._discount_code.DiscountCodeReferenceSchema",
            "inventory-entry": "commercetools._schemas._inventory.InventoryEntryReferenceSchema",
            "order-edit": "commercetools._schemas._order_edit.OrderEditReferenceSchema",
            "order": "commercetools._schemas._order.OrderReferenceSchema",
            "payment": "commercetools._schemas._payment.PaymentReferenceSchema",
            "product-discount": "commercetools._schemas._product_discount.ProductDiscountReferenceSchema",
            "product-type": "commercetools._schemas._product_type.ProductTypeReferenceSchema",
            "product": "commercetools._schemas._product.ProductReferenceSchema",
            "review": "commercetools._schemas._review.ReviewReferenceSchema",
            "shipping-method": "commercetools._schemas._shipping_method.ShippingMethodReferenceSchema",
            "shopping-list": "commercetools._schemas._shopping_list.ShoppingListReferenceSchema",
            "state": "commercetools._schemas._state.StateReferenceSchema",
            "store": "commercetools._schemas._store.StoreReferenceSchema",
            "tax-category": "commercetools._schemas._tax_category.TaxCategoryReferenceSchema",
            "type": "commercetools._schemas._type.TypeReferenceSchema",
            "zone": "commercetools._schemas._zone.ZoneReferenceSchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ExtensionInput(**data)


class ExtensionPagedQueryResponseSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ExtensionPagedQueryResponse`."""

    limit = marshmallow.fields.Integer(allow_none=True)
    count = marshmallow.fields.Integer(allow_none=True)
    total = marshmallow.fields.Integer(allow_none=True, missing=None)
    offset = marshmallow.fields.Integer(allow_none=True)
    results = helpers.LazyNestedField(
        nested="commercetools._schemas._extension.ExtensionSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ExtensionPagedQueryResponse(**data)


class ExtensionSchema(BaseResourceSchema):
    """Marshmallow schema for :class:`commercetools.types.Extension`."""

    id = marshmallow.fields.String(allow_none=True)
    version = marshmallow.fields.Integer(allow_none=True)
    created_at = marshmallow.fields.DateTime(allow_none=True, data_key="createdAt")
    last_modified_at = marshmallow.fields.DateTime(
        allow_none=True, data_key="lastModifiedAt"
    )
    last_modified_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.LastModifiedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="lastModifiedBy",
    )
    created_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.CreatedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="createdBy",
    )
    key = marshmallow.fields.String(allow_none=True, missing=None)
    destination = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "AWSLambda": "commercetools._schemas._extension.ExtensionAWSLambdaDestinationSchema",
            "HTTP": "commercetools._schemas._extension.ExtensionHttpDestinationSchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    triggers = helpers.LazyNestedField(
        nested="commercetools._schemas._extension.ExtensionTriggerSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )
    timeout_in_ms = marshmallow.fields.Integer(
        allow_none=True, missing=None, data_key="timeoutInMs"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.Extension(**data)


class ExtensionTriggerSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ExtensionTrigger`."""

    resource_type_id = marshmallow_enum.EnumField(
        types.ExtensionResourceTypeId, by_value=True, data_key="resourceTypeId"
    )
    actions = marshmallow.fields.List(
        marshmallow_enum.EnumField(types.ExtensionAction, by_value=True)
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ExtensionTrigger(**data)


class ExtensionUpdateActionSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ExtensionUpdateAction`."""

    action = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ExtensionUpdateAction(**data)


class ExtensionUpdateSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ExtensionUpdate`."""

    version = marshmallow.fields.Integer(allow_none=True)
    actions = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("action", "action"),
            discriminator_schemas={
                "changeDestination": "commercetools._schemas._extension.ExtensionChangeDestinationActionSchema",
                "changeTriggers": "commercetools._schemas._extension.ExtensionChangeTriggersActionSchema",
                "setKey": "commercetools._schemas._extension.ExtensionSetKeyActionSchema",
                "setTimeoutInMs": "commercetools._schemas._extension.ExtensionSetTimeoutInMsActionSchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        ),
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ExtensionUpdate(**data)


class ExtensionAWSLambdaDestinationSchema(ExtensionDestinationSchema):
    """Marshmallow schema for :class:`commercetools.types.ExtensionAWSLambdaDestination`."""

    arn = marshmallow.fields.String(allow_none=True)
    access_key = marshmallow.fields.String(allow_none=True, data_key="accessKey")
    access_secret = marshmallow.fields.String(allow_none=True, data_key="accessSecret")

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.ExtensionAWSLambdaDestination(**data)


class ExtensionAuthorizationHeaderAuthenticationSchema(
    ExtensionHttpDestinationAuthenticationSchema
):
    """Marshmallow schema for :class:`commercetools.types.ExtensionAuthorizationHeaderAuthentication`."""

    header_value = marshmallow.fields.String(allow_none=True, data_key="headerValue")

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.ExtensionAuthorizationHeaderAuthentication(**data)


class ExtensionAzureFunctionsAuthenticationSchema(
    ExtensionHttpDestinationAuthenticationSchema
):
    """Marshmallow schema for :class:`commercetools.types.ExtensionAzureFunctionsAuthentication`."""

    key = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.ExtensionAzureFunctionsAuthentication(**data)


class ExtensionChangeDestinationActionSchema(ExtensionUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ExtensionChangeDestinationAction`."""

    destination = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "AWSLambda": "commercetools._schemas._extension.ExtensionAWSLambdaDestinationSchema",
            "HTTP": "commercetools._schemas._extension.ExtensionHttpDestinationSchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ExtensionChangeDestinationAction(**data)


class ExtensionChangeTriggersActionSchema(ExtensionUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ExtensionChangeTriggersAction`."""

    triggers = helpers.LazyNestedField(
        nested="commercetools._schemas._extension.ExtensionTriggerSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ExtensionChangeTriggersAction(**data)


class ExtensionHttpDestinationSchema(ExtensionDestinationSchema):
    """Marshmallow schema for :class:`commercetools.types.ExtensionHttpDestination`."""

    url = marshmallow.fields.String(allow_none=True)
    authentication = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "AuthorizationHeader": "commercetools._schemas._extension.ExtensionAuthorizationHeaderAuthenticationSchema",
            "AzureFunctions": "commercetools._schemas._extension.ExtensionAzureFunctionsAuthenticationSchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.ExtensionHttpDestination(**data)


class ExtensionSetKeyActionSchema(ExtensionUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ExtensionSetKeyAction`."""

    key = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ExtensionSetKeyAction(**data)


class ExtensionSetTimeoutInMsActionSchema(ExtensionUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ExtensionSetTimeoutInMsAction`."""

    timeout_in_ms = marshmallow.fields.Integer(
        allow_none=True, missing=None, data_key="timeoutInMs"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ExtensionSetTimeoutInMsAction(**data)



# DO NOT EDIT! This file is automatically generated
import marshmallow

from commercetools import helpers, types
from commercetools._schemas._common import (
    BaseResourceSchema,
    LocalizedStringField,
    ReferenceSchema,
    ResourceIdentifierSchema,
)
from commercetools._schemas._type import FieldContainerField

__all__ = [
    "MyShoppingListSchema",
    "ShoppingListAddLineItemActionSchema",
    "ShoppingListAddTextLineItemActionSchema",
    "ShoppingListChangeLineItemQuantityActionSchema",
    "ShoppingListChangeLineItemsOrderActionSchema",
    "ShoppingListChangeNameActionSchema",
    "ShoppingListChangeTextLineItemNameActionSchema",
    "ShoppingListChangeTextLineItemQuantityActionSchema",
    "ShoppingListChangeTextLineItemsOrderActionSchema",
    "ShoppingListDraftSchema",
    "ShoppingListLineItemDraftSchema",
    "ShoppingListLineItemSchema",
    "ShoppingListPagedQueryResponseSchema",
    "ShoppingListReferenceSchema",
    "ShoppingListRemoveLineItemActionSchema",
    "ShoppingListRemoveTextLineItemActionSchema",
    "ShoppingListResourceIdentifierSchema",
    "ShoppingListSchema",
    "ShoppingListSetAnonymousIdActionSchema",
    "ShoppingListSetCustomFieldActionSchema",
    "ShoppingListSetCustomTypeActionSchema",
    "ShoppingListSetCustomerActionSchema",
    "ShoppingListSetDeleteDaysAfterLastModificationActionSchema",
    "ShoppingListSetDescriptionActionSchema",
    "ShoppingListSetKeyActionSchema",
    "ShoppingListSetLineItemCustomFieldActionSchema",
    "ShoppingListSetLineItemCustomTypeActionSchema",
    "ShoppingListSetSlugActionSchema",
    "ShoppingListSetTextLineItemCustomFieldActionSchema",
    "ShoppingListSetTextLineItemCustomTypeActionSchema",
    "ShoppingListSetTextLineItemDescriptionActionSchema",
    "ShoppingListUpdateActionSchema",
    "ShoppingListUpdateSchema",
    "TextLineItemDraftSchema",
    "TextLineItemSchema",
]


class MyShoppingListSchema(BaseResourceSchema):
    """Marshmallow schema for :class:`commercetools.types.MyShoppingList`."""

    id = marshmallow.fields.String(allow_none=True)
    version = marshmallow.fields.Integer(allow_none=True)
    created_at = marshmallow.fields.DateTime(allow_none=True, data_key="createdAt")
    last_modified_at = marshmallow.fields.DateTime(
        allow_none=True, data_key="lastModifiedAt"
    )
    last_modified_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.LastModifiedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="lastModifiedBy",
    )
    created_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.CreatedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="createdBy",
    )
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    customer = helpers.LazyNestedField(
        nested="commercetools._schemas._customer.CustomerReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    delete_days_after_last_modification = marshmallow.fields.Integer(
        allow_none=True, missing=None, data_key="deleteDaysAfterLastModification"
    )
    description = LocalizedStringField(allow_none=True, missing=None)
    key = marshmallow.fields.String(allow_none=True, missing=None)
    line_items = helpers.LazyNestedField(
        nested="commercetools._schemas._shopping_list.ShoppingListLineItemSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
        data_key="lineItems",
    )
    name = LocalizedStringField(allow_none=True)
    slug = LocalizedStringField(allow_none=True, missing=None)
    text_line_items = helpers.LazyNestedField(
        nested="commercetools._schemas._shopping_list.TextLineItemSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
        data_key="textLineItems",
    )
    anonymous_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="anonymousId"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.MyShoppingList(**data)


class ShoppingListDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListDraft`."""

    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    customer = helpers.LazyNestedField(
        nested="commercetools._schemas._customer.CustomerResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    delete_days_after_last_modification = marshmallow.fields.Integer(
        allow_none=True, missing=None, data_key="deleteDaysAfterLastModification"
    )
    description = LocalizedStringField(allow_none=True, missing=None)
    key = marshmallow.fields.String(allow_none=True, missing=None)
    line_items = helpers.LazyNestedField(
        nested="commercetools._schemas._shopping_list.ShoppingListLineItemDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
        data_key="lineItems",
    )
    name = LocalizedStringField(allow_none=True)
    slug = LocalizedStringField(allow_none=True, missing=None)
    text_line_items = helpers.LazyNestedField(
        nested="commercetools._schemas._shopping_list.TextLineItemDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
        data_key="textLineItems",
    )
    anonymous_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="anonymousId"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ShoppingListDraft(**data)


class ShoppingListLineItemDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListLineItemDraft`."""

    added_at = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="addedAt"
    )
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    sku = marshmallow.fields.String(allow_none=True, missing=None)
    product_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="productId"
    )
    quantity = marshmallow.fields.Integer(allow_none=True, missing=None)
    variant_id = marshmallow.fields.Integer(
        allow_none=True, missing=None, data_key="variantId"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ShoppingListLineItemDraft(**data)


class ShoppingListLineItemSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListLineItem`."""

    added_at = marshmallow.fields.DateTime(allow_none=True, data_key="addedAt")
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    deactivated_at = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="deactivatedAt"
    )
    id = marshmallow.fields.String(allow_none=True)
    name = LocalizedStringField(allow_none=True)
    product_id = marshmallow.fields.String(allow_none=True, data_key="productId")
    product_slug = LocalizedStringField(
        allow_none=True, missing=None, data_key="productSlug"
    )
    product_type = helpers.LazyNestedField(
        nested="commercetools._schemas._product_type.ProductTypeReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="productType",
    )
    quantity = marshmallow.fields.Integer(allow_none=True)
    variant = helpers.LazyNestedField(
        nested="commercetools._schemas._product.ProductVariantSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    variant_id = marshmallow.fields.Integer(
        allow_none=True, missing=None, data_key="variantId"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ShoppingListLineItem(**data)


class ShoppingListPagedQueryResponseSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListPagedQueryResponse`."""

    limit = marshmallow.fields.Integer(allow_none=True)
    count = marshmallow.fields.Integer(allow_none=True)
    total = marshmallow.fields.Integer(allow_none=True, missing=None)
    offset = marshmallow.fields.Integer(allow_none=True)
    results = helpers.LazyNestedField(
        nested="commercetools._schemas._shopping_list.ShoppingListSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ShoppingListPagedQueryResponse(**data)


class ShoppingListReferenceSchema(ReferenceSchema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListReference`."""

    obj = helpers.LazyNestedField(
        nested="commercetools._schemas._shopping_list.ShoppingListSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.ShoppingListReference(**data)


class ShoppingListResourceIdentifierSchema(ResourceIdentifierSchema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListResourceIdentifier`."""

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.ShoppingListResourceIdentifier(**data)


class ShoppingListSchema(BaseResourceSchema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingList`."""

    id = marshmallow.fields.String(allow_none=True)
    version = marshmallow.fields.Integer(allow_none=True)
    created_at = marshmallow.fields.DateTime(allow_none=True, data_key="createdAt")
    last_modified_at = marshmallow.fields.DateTime(
        allow_none=True, data_key="lastModifiedAt"
    )
    last_modified_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.LastModifiedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="lastModifiedBy",
    )
    created_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.CreatedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="createdBy",
    )
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    customer = helpers.LazyNestedField(
        nested="commercetools._schemas._customer.CustomerReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    delete_days_after_last_modification = marshmallow.fields.Integer(
        allow_none=True, missing=None, data_key="deleteDaysAfterLastModification"
    )
    description = LocalizedStringField(allow_none=True, missing=None)
    key = marshmallow.fields.String(allow_none=True, missing=None)
    line_items = helpers.LazyNestedField(
        nested="commercetools._schemas._shopping_list.ShoppingListLineItemSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
        data_key="lineItems",
    )
    name = LocalizedStringField(allow_none=True)
    slug = LocalizedStringField(allow_none=True, missing=None)
    text_line_items = helpers.LazyNestedField(
        nested="commercetools._schemas._shopping_list.TextLineItemSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
        data_key="textLineItems",
    )
    anonymous_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="anonymousId"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ShoppingList(**data)


class ShoppingListUpdateActionSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListUpdateAction`."""

    action = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShoppingListUpdateAction(**data)


class ShoppingListUpdateSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListUpdate`."""

    version = marshmallow.fields.Integer(allow_none=True)
    actions = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("action", "action"),
            discriminator_schemas={
                "addLineItem": "commercetools._schemas._shopping_list.ShoppingListAddLineItemActionSchema",
                "addTextLineItem": "commercetools._schemas._shopping_list.ShoppingListAddTextLineItemActionSchema",
                "changeLineItemQuantity": "commercetools._schemas._shopping_list.ShoppingListChangeLineItemQuantityActionSchema",
                "changeLineItemsOrder": "commercetools._schemas._shopping_list.ShoppingListChangeLineItemsOrderActionSchema",
                "changeName": "commercetools._schemas._shopping_list.ShoppingListChangeNameActionSchema",
                "changeTextLineItemName": "commercetools._schemas._shopping_list.ShoppingListChangeTextLineItemNameActionSchema",
                "changeTextLineItemQuantity": "commercetools._schemas._shopping_list.ShoppingListChangeTextLineItemQuantityActionSchema",
                "changeTextLineItemsOrder": "commercetools._schemas._shopping_list.ShoppingListChangeTextLineItemsOrderActionSchema",
                "removeLineItem": "commercetools._schemas._shopping_list.ShoppingListRemoveLineItemActionSchema",
                "removeTextLineItem": "commercetools._schemas._shopping_list.ShoppingListRemoveTextLineItemActionSchema",
                "setAnonymousId": "commercetools._schemas._shopping_list.ShoppingListSetAnonymousIdActionSchema",
                "setCustomField": "commercetools._schemas._shopping_list.ShoppingListSetCustomFieldActionSchema",
                "setCustomType": "commercetools._schemas._shopping_list.ShoppingListSetCustomTypeActionSchema",
                "setCustomer": "commercetools._schemas._shopping_list.ShoppingListSetCustomerActionSchema",
                "setDeleteDaysAfterLastModification": "commercetools._schemas._shopping_list.ShoppingListSetDeleteDaysAfterLastModificationActionSchema",
                "setDescription": "commercetools._schemas._shopping_list.ShoppingListSetDescriptionActionSchema",
                "setKey": "commercetools._schemas._shopping_list.ShoppingListSetKeyActionSchema",
                "setLineItemCustomField": "commercetools._schemas._shopping_list.ShoppingListSetLineItemCustomFieldActionSchema",
                "setLineItemCustomType": "commercetools._schemas._shopping_list.ShoppingListSetLineItemCustomTypeActionSchema",
                "setSlug": "commercetools._schemas._shopping_list.ShoppingListSetSlugActionSchema",
                "setTextLineItemCustomField": "commercetools._schemas._shopping_list.ShoppingListSetTextLineItemCustomFieldActionSchema",
                "setTextLineItemCustomType": "commercetools._schemas._shopping_list.ShoppingListSetTextLineItemCustomTypeActionSchema",
                "setTextLineItemDescription": "commercetools._schemas._shopping_list.ShoppingListSetTextLineItemDescriptionActionSchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        ),
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ShoppingListUpdate(**data)


class TextLineItemDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.TextLineItemDraft`."""

    added_at = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="addedAt"
    )
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    description = LocalizedStringField(allow_none=True, missing=None)
    name = LocalizedStringField(allow_none=True)
    quantity = marshmallow.fields.Integer(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.TextLineItemDraft(**data)


class TextLineItemSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.TextLineItem`."""

    added_at = marshmallow.fields.DateTime(allow_none=True, data_key="addedAt")
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    description = LocalizedStringField(allow_none=True, missing=None)
    id = marshmallow.fields.String(allow_none=True)
    name = LocalizedStringField(allow_none=True)
    quantity = marshmallow.fields.Integer(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.TextLineItem(**data)


class ShoppingListAddLineItemActionSchema(ShoppingListUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListAddLineItemAction`."""

    sku = marshmallow.fields.String(allow_none=True, missing=None)
    product_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="productId"
    )
    variant_id = marshmallow.fields.Integer(
        allow_none=True, missing=None, data_key="variantId"
    )
    quantity = marshmallow.fields.Integer(allow_none=True, missing=None)
    added_at = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="addedAt"
    )
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShoppingListAddLineItemAction(**data)


class ShoppingListAddTextLineItemActionSchema(ShoppingListUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListAddTextLineItemAction`."""

    name = LocalizedStringField(allow_none=True)
    description = LocalizedStringField(allow_none=True, missing=None)
    quantity = marshmallow.fields.Integer(allow_none=True, missing=None)
    added_at = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="addedAt"
    )
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShoppingListAddTextLineItemAction(**data)


class ShoppingListChangeLineItemQuantityActionSchema(ShoppingListUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListChangeLineItemQuantityAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    quantity = marshmallow.fields.Integer(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShoppingListChangeLineItemQuantityAction(**data)


class ShoppingListChangeLineItemsOrderActionSchema(ShoppingListUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListChangeLineItemsOrderAction`."""

    line_item_order = marshmallow.fields.List(
        marshmallow.fields.String(allow_none=True), data_key="lineItemOrder"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShoppingListChangeLineItemsOrderAction(**data)


class ShoppingListChangeNameActionSchema(ShoppingListUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListChangeNameAction`."""

    name = LocalizedStringField(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShoppingListChangeNameAction(**data)


class ShoppingListChangeTextLineItemNameActionSchema(ShoppingListUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListChangeTextLineItemNameAction`."""

    text_line_item_id = marshmallow.fields.String(
        allow_none=True, data_key="textLineItemId"
    )
    name = LocalizedStringField(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShoppingListChangeTextLineItemNameAction(**data)


class ShoppingListChangeTextLineItemQuantityActionSchema(
    ShoppingListUpdateActionSchema
):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListChangeTextLineItemQuantityAction`."""

    text_line_item_id = marshmallow.fields.String(
        allow_none=True, data_key="textLineItemId"
    )
    quantity = marshmallow.fields.Integer(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShoppingListChangeTextLineItemQuantityAction(**data)


class ShoppingListChangeTextLineItemsOrderActionSchema(ShoppingListUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListChangeTextLineItemsOrderAction`."""

    text_line_item_order = marshmallow.fields.List(
        marshmallow.fields.String(allow_none=True), data_key="textLineItemOrder"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShoppingListChangeTextLineItemsOrderAction(**data)


class ShoppingListRemoveLineItemActionSchema(ShoppingListUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListRemoveLineItemAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    quantity = marshmallow.fields.Integer(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShoppingListRemoveLineItemAction(**data)


class ShoppingListRemoveTextLineItemActionSchema(ShoppingListUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListRemoveTextLineItemAction`."""

    text_line_item_id = marshmallow.fields.String(
        allow_none=True, data_key="textLineItemId"
    )
    quantity = marshmallow.fields.Integer(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShoppingListRemoveTextLineItemAction(**data)


class ShoppingListSetAnonymousIdActionSchema(ShoppingListUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListSetAnonymousIdAction`."""

    anonymous_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="anonymousId"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShoppingListSetAnonymousIdAction(**data)


class ShoppingListSetCustomFieldActionSchema(ShoppingListUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListSetCustomFieldAction`."""

    name = marshmallow.fields.String(allow_none=True)
    value = marshmallow.fields.Raw(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShoppingListSetCustomFieldAction(**data)


class ShoppingListSetCustomTypeActionSchema(ShoppingListUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListSetCustomTypeAction`."""

    type = helpers.LazyNestedField(
        nested="commercetools._schemas._type.TypeResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    fields = FieldContainerField(allow_none=True, missing=None)  # type: ignore

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShoppingListSetCustomTypeAction(**data)


class ShoppingListSetCustomerActionSchema(ShoppingListUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListSetCustomerAction`."""

    customer = helpers.LazyNestedField(
        nested="commercetools._schemas._customer.CustomerResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShoppingListSetCustomerAction(**data)


class ShoppingListSetDeleteDaysAfterLastModificationActionSchema(
    ShoppingListUpdateActionSchema
):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListSetDeleteDaysAfterLastModificationAction`."""

    delete_days_after_last_modification = marshmallow.fields.Integer(
        allow_none=True, missing=None, data_key="deleteDaysAfterLastModification"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShoppingListSetDeleteDaysAfterLastModificationAction(**data)


class ShoppingListSetDescriptionActionSchema(ShoppingListUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListSetDescriptionAction`."""

    description = LocalizedStringField(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShoppingListSetDescriptionAction(**data)


class ShoppingListSetKeyActionSchema(ShoppingListUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListSetKeyAction`."""

    key = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShoppingListSetKeyAction(**data)


class ShoppingListSetLineItemCustomFieldActionSchema(ShoppingListUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListSetLineItemCustomFieldAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    name = marshmallow.fields.String(allow_none=True)
    value = marshmallow.fields.Raw(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShoppingListSetLineItemCustomFieldAction(**data)


class ShoppingListSetLineItemCustomTypeActionSchema(ShoppingListUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListSetLineItemCustomTypeAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    type = helpers.LazyNestedField(
        nested="commercetools._schemas._type.TypeResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    fields = FieldContainerField(allow_none=True, missing=None)  # type: ignore

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShoppingListSetLineItemCustomTypeAction(**data)


class ShoppingListSetSlugActionSchema(ShoppingListUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListSetSlugAction`."""

    slug = LocalizedStringField(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShoppingListSetSlugAction(**data)


class ShoppingListSetTextLineItemCustomFieldActionSchema(
    ShoppingListUpdateActionSchema
):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListSetTextLineItemCustomFieldAction`."""

    text_line_item_id = marshmallow.fields.String(
        allow_none=True, data_key="textLineItemId"
    )
    name = marshmallow.fields.String(allow_none=True)
    value = marshmallow.fields.Raw(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShoppingListSetTextLineItemCustomFieldAction(**data)


class ShoppingListSetTextLineItemCustomTypeActionSchema(ShoppingListUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListSetTextLineItemCustomTypeAction`."""

    text_line_item_id = marshmallow.fields.String(
        allow_none=True, data_key="textLineItemId"
    )
    type = helpers.LazyNestedField(
        nested="commercetools._schemas._type.TypeResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    fields = FieldContainerField(allow_none=True, missing=None)  # type: ignore

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShoppingListSetTextLineItemCustomTypeAction(**data)


class ShoppingListSetTextLineItemDescriptionActionSchema(
    ShoppingListUpdateActionSchema
):
    """Marshmallow schema for :class:`commercetools.types.ShoppingListSetTextLineItemDescriptionAction`."""

    text_line_item_id = marshmallow.fields.String(
        allow_none=True, data_key="textLineItemId"
    )
    description = LocalizedStringField(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShoppingListSetTextLineItemDescriptionAction(**data)



# DO NOT EDIT! This file is automatically generated
import marshmallow

from commercetools import helpers, types
from commercetools._schemas._common import (
    BaseResourceSchema,
    ReferenceSchema,
    ResourceIdentifierSchema,
)
from commercetools._schemas._type import FieldContainerField

__all__ = [
    "InventoryEntryAddQuantityActionSchema",
    "InventoryEntryChangeQuantityActionSchema",
    "InventoryEntryDraftSchema",
    "InventoryEntryReferenceSchema",
    "InventoryEntryRemoveQuantityActionSchema",
    "InventoryEntryResourceIdentifierSchema",
    "InventoryEntrySchema",
    "InventoryEntrySetCustomFieldActionSchema",
    "InventoryEntrySetCustomTypeActionSchema",
    "InventoryEntrySetExpectedDeliveryActionSchema",
    "InventoryEntrySetRestockableInDaysActionSchema",
    "InventoryEntrySetSupplyChannelActionSchema",
    "InventoryEntryUpdateActionSchema",
    "InventoryEntryUpdateSchema",
    "InventoryPagedQueryResponseSchema",
]


class InventoryEntryDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.InventoryEntryDraft`."""

    sku = marshmallow.fields.String(allow_none=True)
    supply_channel = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="supplyChannel",
    )
    quantity_on_stock = marshmallow.fields.Integer(
        allow_none=True, data_key="quantityOnStock"
    )
    restockable_in_days = marshmallow.fields.Integer(
        allow_none=True, missing=None, data_key="restockableInDays"
    )
    expected_delivery = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="expectedDelivery"
    )
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.InventoryEntryDraft(**data)


class InventoryEntryReferenceSchema(ReferenceSchema):
    """Marshmallow schema for :class:`commercetools.types.InventoryEntryReference`."""

    obj = helpers.LazyNestedField(
        nested="commercetools._schemas._inventory.InventoryEntrySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.InventoryEntryReference(**data)


class InventoryEntryResourceIdentifierSchema(ResourceIdentifierSchema):
    """Marshmallow schema for :class:`commercetools.types.InventoryEntryResourceIdentifier`."""

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.InventoryEntryResourceIdentifier(**data)


class InventoryEntrySchema(BaseResourceSchema):
    """Marshmallow schema for :class:`commercetools.types.InventoryEntry`."""

    id = marshmallow.fields.String(allow_none=True)
    version = marshmallow.fields.Integer(allow_none=True)
    created_at = marshmallow.fields.DateTime(allow_none=True, data_key="createdAt")
    last_modified_at = marshmallow.fields.DateTime(
        allow_none=True, data_key="lastModifiedAt"
    )
    last_modified_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.LastModifiedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="lastModifiedBy",
    )
    created_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.CreatedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="createdBy",
    )
    sku = marshmallow.fields.String(allow_none=True)
    supply_channel = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="supplyChannel",
    )
    quantity_on_stock = marshmallow.fields.Integer(
        allow_none=True, data_key="quantityOnStock"
    )
    available_quantity = marshmallow.fields.Integer(
        allow_none=True, data_key="availableQuantity"
    )
    restockable_in_days = marshmallow.fields.Integer(
        allow_none=True, missing=None, data_key="restockableInDays"
    )
    expected_delivery = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="expectedDelivery"
    )
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.InventoryEntry(**data)


class InventoryEntryUpdateActionSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.InventoryEntryUpdateAction`."""

    action = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.InventoryEntryUpdateAction(**data)


class InventoryEntryUpdateSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.InventoryEntryUpdate`."""

    version = marshmallow.fields.Integer(allow_none=True)
    actions = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("action", "action"),
            discriminator_schemas={
                "addQuantity": "commercetools._schemas._inventory.InventoryEntryAddQuantityActionSchema",
                "changeQuantity": "commercetools._schemas._inventory.InventoryEntryChangeQuantityActionSchema",
                "removeQuantity": "commercetools._schemas._inventory.InventoryEntryRemoveQuantityActionSchema",
                "setCustomField": "commercetools._schemas._inventory.InventoryEntrySetCustomFieldActionSchema",
                "setCustomType": "commercetools._schemas._inventory.InventoryEntrySetCustomTypeActionSchema",
                "setExpectedDelivery": "commercetools._schemas._inventory.InventoryEntrySetExpectedDeliveryActionSchema",
                "setRestockableInDays": "commercetools._schemas._inventory.InventoryEntrySetRestockableInDaysActionSchema",
                "setSupplyChannel": "commercetools._schemas._inventory.InventoryEntrySetSupplyChannelActionSchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        ),
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.InventoryEntryUpdate(**data)


class InventoryPagedQueryResponseSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.InventoryPagedQueryResponse`."""

    limit = marshmallow.fields.Integer(allow_none=True)
    count = marshmallow.fields.Integer(allow_none=True)
    total = marshmallow.fields.Integer(allow_none=True, missing=None)
    offset = marshmallow.fields.Integer(allow_none=True)
    results = helpers.LazyNestedField(
        nested="commercetools._schemas._inventory.InventoryEntrySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.InventoryPagedQueryResponse(**data)


class InventoryEntryAddQuantityActionSchema(InventoryEntryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.InventoryEntryAddQuantityAction`."""

    quantity = marshmallow.fields.Integer(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.InventoryEntryAddQuantityAction(**data)


class InventoryEntryChangeQuantityActionSchema(InventoryEntryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.InventoryEntryChangeQuantityAction`."""

    quantity = marshmallow.fields.Integer(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.InventoryEntryChangeQuantityAction(**data)


class InventoryEntryRemoveQuantityActionSchema(InventoryEntryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.InventoryEntryRemoveQuantityAction`."""

    quantity = marshmallow.fields.Integer(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.InventoryEntryRemoveQuantityAction(**data)


class InventoryEntrySetCustomFieldActionSchema(InventoryEntryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.InventoryEntrySetCustomFieldAction`."""

    name = marshmallow.fields.String(allow_none=True)
    value = marshmallow.fields.Raw(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.InventoryEntrySetCustomFieldAction(**data)


class InventoryEntrySetCustomTypeActionSchema(InventoryEntryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.InventoryEntrySetCustomTypeAction`."""

    type = helpers.LazyNestedField(
        nested="commercetools._schemas._type.TypeResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    fields = FieldContainerField(allow_none=True, missing=None)  # type: ignore

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.InventoryEntrySetCustomTypeAction(**data)


class InventoryEntrySetExpectedDeliveryActionSchema(InventoryEntryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.InventoryEntrySetExpectedDeliveryAction`."""

    expected_delivery = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="expectedDelivery"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.InventoryEntrySetExpectedDeliveryAction(**data)


class InventoryEntrySetRestockableInDaysActionSchema(InventoryEntryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.InventoryEntrySetRestockableInDaysAction`."""

    restockable_in_days = marshmallow.fields.Integer(
        allow_none=True, missing=None, data_key="restockableInDays"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.InventoryEntrySetRestockableInDaysAction(**data)


class InventoryEntrySetSupplyChannelActionSchema(InventoryEntryUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.InventoryEntrySetSupplyChannelAction`."""

    supply_channel = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="supplyChannel",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.InventoryEntrySetSupplyChannelAction(**data)
